@@ -3,6 +3,7 @@ import itertools
 import json
 import math
 import time
+import typing
 from dataclasses import dataclass
 from datetime import datetime
 from datetime import timezone
@@ -36,6 +37,7 @@ from reader import ParseError
 from reader import ReaderError
 from reader._plugins import Loader
 from reader.types import _get_entry_content
+from reader.types import TristateFilterInput
 
 
 blueprint = Blueprint('reader', __name__)
@@ -75,7 +77,7 @@ def log_scale(n, p=2):
     # https://math.stackexchange.com/a/970251
     # https://math.stackexchange.com/a/3428961
     n = float(n)
-    c = 10 ** p
+    c = 10**p
     return math.log10(n * c + 1) / math.log10(c + 1)
 
 
@@ -204,8 +206,9 @@ def entries():
     has_enclosures = request.args.get('has-enclosures')
     has_enclosures = {None: None, 'no': False, 'yes': True}[has_enclosures]
 
-    important = request.args.get('important')
-    important = {None: None, 'no': False, 'yes': True}[important]
+    important = request.args.get('important', 'notfalse')
+    if important not in typing.get_args(TristateFilterInput):
+        abort(400, f"invalid important: {important}")
 
     if not request.args.get('q'):
         sort = request.args.get('sort', 'recent')
@@ -593,29 +596,15 @@ def mark_as_read(data):
 def mark_as_unread(data):
     feed_url = data['feed-url']
     entry_id = data['entry-id']
-    id = feed_url, entry_id
-    reader = get_reader()
-    reader.mark_entry_as_unread(id)
-    entry = reader.get_entry(id)
-    # if we're in "don't care", reset important_modified
-    if not entry.important and entry.important_modified:
-        reader.set_entry_important(id, False, modified=None)
-
-
-@form_api
-@readererror_to_apierror()
-def mark_as_dont_care(data):
-    feed_url = data['feed-url']
-    entry_id = data['entry-id']
-    get_reader()._mark_entry_as_dont_care((feed_url, entry_id))
+    get_reader().mark_entry_as_unread((feed_url, entry_id))
 
 
 @form_api(really=True)
 @readererror_to_apierror()
 def mark_all_as_read(data):
     feed_url = data['feed-url']
-    entry_id = json.loads(data['entry-id'])
-    for entry_id in entry_id:
+    entry_ids = json.loads(data['entry-id'])
+    for entry_id in entry_ids:
         get_reader().mark_entry_as_read((feed_url, entry_id))
 
 
@@ -623,8 +612,8 @@ def mark_all_as_read(data):
 @readererror_to_apierror()
 def mark_all_as_unread(data):
     feed_url = data['feed-url']
-    entry_id = json.loads(data['entry-id'])
-    for entry_id in entry_id:
+    entry_ids = json.loads(data['entry-id'])
+    for entry_id in entry_ids:
         get_reader().mark_entry_as_unread((feed_url, entry_id))
 
 
@@ -636,13 +625,20 @@ def mark_as_important(data):
     get_reader().mark_entry_as_important((feed_url, entry_id))
 
 
+@form_api
+@readererror_to_apierror()
+def clear_important(data):
+    feed_url = data['feed-url']
+    entry_id = data['entry-id']
+    get_reader().set_entry_important((feed_url, entry_id), None)
+
+
 @form_api
 @readererror_to_apierror()
 def mark_as_unimportant(data):
     feed_url = data['feed-url']
     entry_id = data['entry-id']
-    # only "don't care" sets important_modified
-    get_reader().set_entry_important((feed_url, entry_id), False, modified=None)
+    get_reader().mark_entry_as_unimportant((feed_url, entry_id))
 
 
 @form_api(really=True)
@@ -694,7 +690,7 @@ def update_metadata(data):
     try:
         value = yaml.safe_load(data['value'])
     except yaml.YAMLError as e:
-        raise APIError("invalid JSON: {}".format(e), resource_id + (key,)) from e
+        raise APIError(f"invalid JSON: {e}", resource_id + (key,)) from e
     get_reader().set_tag(resource_id, key, value)
 
 
