@@ -2,6 +2,7 @@ import functools
 import json
 import logging
 import os.path
+import shutil
 import sys
 import traceback
 from contextlib import nullcontext
@@ -11,7 +12,6 @@ import click
 import yaml
 
 import reader
-from . import ReaderError
 from . import StorageError
 from ._config import make_reader_config
 from ._config import make_reader_from_config
@@ -132,9 +132,8 @@ def log_command(fn):
                 e,
                 exc_info=True,
             )
-            if not isinstance(e, ReaderError):
-                raise
-            click.get_current_context().exit(1)
+            # always raise, even if it's ReaderError (it could be due to a bug)
+            raise
 
     return wrapper
 
@@ -298,10 +297,16 @@ def iter_update_status(it, length):
 
         if result.not_modified:
             status = 'not modified'
+            if result.updated_feed:
+                status += f", {result.value.total} total"
         elif result.error:
             status = red(result.error)
         else:
-            status = green(f"{result.value.new} new, {result.value.modified} modified")
+            status = green(
+                f"{result.value.new} new, "
+                f"{result.value.modified} modified, "
+                f"{result.value.total} total"
+            )
 
         click.echo(f"{elapsed}\t{pos}\t{result.url}\t{status}")
 
@@ -350,7 +355,7 @@ def update(reader, url, new_only, workers, verbose):
 
     def feed_stats(width=None):
         if not width:
-            width, _ = click.get_terminal_size()
+            width, _ = shutil.get_terminal_size()
         if width < 80:
             return ''
         if width < 105:
@@ -416,7 +421,7 @@ def entries(reader):
 
     """
     for entry in reader.get_entries():
-        click.echo("{} {}".format(entry.feed.url, entry.link or entry.id))
+        click.echo(f"{entry.feed.url} {entry.link or entry.id}")
 
 
 @cli.group()
@@ -467,7 +472,7 @@ def search_entries(reader, query):
     """
     for rv in reader.search_entries(query):
         entry = reader.get_entry(rv)
-        click.echo("{} {}".format(entry.feed.url, entry.link or entry.id))
+        click.echo(f"{entry.feed.url} {entry.link or entry.id}")
 
 
 @cli.group()
