@@ -55,6 +55,7 @@ Screenshots:
 
 To do (roughly in order of importance):
 
+* stale feeds fail update
 * retrieve media/polls in quoted/retweeted tweet
 * media URL might be None
 * retrieve retweets/quotes of retweets/quotes
@@ -63,11 +64,14 @@ To do (roughly in order of importance):
 * lower the initial tweet limit, but allow increasing it
 * automatically re-render entry HTML on plugin update
 * show images / expanded URLs only in the original tweet, not in retweets
+* poll counts are not updated after the first update
 * mark updated entries as unread
 * better URL/entity expansion (feed subtitle, entry hashtags and usernames)
 * better media rendering
 * better poll rendering
+* pull a single thread (a la threader app)
 * retrieve and render tweet replies (``https://twitter.com/user?replies=yes``)
+* handle edit_history_tweet_ids somehow
 * support previewing Twitter feeds in the web app
 
 
@@ -183,6 +187,21 @@ class UserFile(NamedTuple):
     conversations: list[Conversation]
 
 
+def make_tweet(data):
+    """Make a tweepy.Tweet from pre-4.11 data.
+
+    Starting with tweepy 4.11, data must contain edit_history_tweet_ids.
+    tweepy >4.12 allows it to be missing, but warns, which we want to avoid.
+
+    https://docs.tweepy.org/en/stable/v2_models.html#tweepy.Tweet.edit_history_tweet_ids
+
+    """
+    data = dict(data)
+    if 'edit_history_tweet_ids' not in data:
+        data['edit_history_tweet_ids'] = [data["id"]]
+    return tweepy.Tweet(data)
+
+
 @dataclass
 class Conversation:
     """Collection of resources belonging to a conversation."""
@@ -194,7 +213,7 @@ class Conversation:
     polls: dict[str, tweepy.Poll] = field(default_factory=dict)
 
     _factories = dict(
-        tweets=(int, tweepy.Tweet),
+        tweets=(int, make_tweet),
         users=(int, tweepy.User),
         media=(lambda k: k, tweepy.Media),
         polls=(lambda k: k, tweepy.Poll),
@@ -480,8 +499,8 @@ class Parser:
     reader: Reader
     http_accept = MIME_TYPE
 
-    def __call__(self, url, file, headers):
-        user, conversations = file
+    def __call__(self, url, resource, headers):
+        user, conversations = resource
         feed = render_user_feed(url, user)
         entries = (
             EntryData(
