@@ -1,29 +1,24 @@
+from __future__ import annotations
+
 import functools
 import json
 import logging
 import random
 import sqlite3
 import string
-import warnings
 from collections import OrderedDict
+from collections.abc import Iterable
+from collections.abc import Iterator
 from contextlib import contextmanager
 from datetime import datetime
 from functools import partial
 from itertools import groupby
 from types import MappingProxyType
 from typing import Any
-from typing import Dict
-from typing import Iterable
-from typing import Iterator
-from typing import List
-from typing import Optional
-from typing import Tuple
 from typing import TYPE_CHECKING
 from typing import TypeVar
-from typing import Union
-
-import bs4  # type: ignore
 
+from ._html_utils import strip_html as strip_html_str
 from ._sql_utils import paginated_query
 from ._sql_utils import Query
 from ._sqlite_utils import DBError
@@ -57,54 +52,14 @@ if TYPE_CHECKING:  # pragma: no cover
 
 log = logging.getLogger('reader')
 
-
 _T = TypeVar('_T')
 
 
-# BeautifulSoup warns if not giving it a parser explicitly; full text:
-#
-#   No parser was explicitly specified, so I'm using the best available
-#   HTML parser for this system ("..."). This usually isn't a problem,
-#   but if you run this code on another system, or in a different virtual
-#   environment, it may use a different parser and behave differently.
-#
-# We are ok with any parser, and with how BeautifulSoup picks the best one if
-# available. Explicitly using generic features (e.g. `('html', 'fast')`,
-# the default) instead of a specific parser still warns.
-#
-# Currently there's no way to allow users to pick a parser, and we don't want
-# to force a specific parser, so there's no point in warning.
-#
-# When changing this, also change the equivalent pytest.filterwarnings config.
-#
-# TODO: Expose BeautifulSoup(features=...) when we have a config system.
-#
-warnings.filterwarnings(
-    'ignore', message='No parser was explicitly specified', module='reader._search'
-)
-
-
-@functools.lru_cache()
-def strip_html(text: SQLiteType, features: Optional[str] = None) -> SQLiteType:
+@functools.lru_cache
+def strip_html(text: SQLiteType) -> SQLiteType:
     if not isinstance(text, str):
         return text
-
-    soup = bs4.BeautifulSoup(text, features=features)
-
-    # <script>, <noscript> and <style> don't contain things relevant to search.
-    # <title> probably does, but its content should already be in the entry title.
-    #
-    # Although <head> is supposed to contain machine-readable content, Firefox
-    # shows any free-floating text it contains, so we should keep it around.
-    #
-    for e in soup.select('script, noscript, style, title'):
-        e.replace_with('\n')
-
-    rv = soup.get_text(separator=' ')
-    # TODO: Remove this assert once bs4 gets type annotations.
-    assert isinstance(rv, str)
-
-    return rv
+    return strip_html_str(text)
 
 
 _QUERY_ERROR_MESSAGE_FRAGMENTS = [
@@ -637,7 +592,7 @@ class Search:
             # nothing to update
             return False
 
-        stripped: List[Dict[str, Any]] = []
+        stripped: list[dict[str, Any]] = []
         for (
             id,
             feed_url,
@@ -649,7 +604,7 @@ class Search:
             summary,
             content_json,
         ) in rows:
-            final: List[Union[Tuple[str, str], Tuple[None, None]]] = []
+            final: list[tuple[str, str] | tuple[None, None]] = []
 
             content = json.loads(content_json) if content_json else []
             if content and isinstance(content, list):
@@ -799,8 +754,8 @@ class Search:
         now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
         sort: SearchSortOrder = 'relevant',
-        limit: Optional[int] = None,
-        starting_after: Optional[Tuple[str, str]] = None,
+        limit: int | None = None,
+        starting_after: tuple[str, str] | None = None,
     ) -> Iterable[EntrySearchResult]:
         # TODO: dupe of at least Storage.get_entries(), maybe deduplicate
         if sort in ('relevant', 'recent'):
@@ -837,7 +792,7 @@ class Search:
 
     @wrap_exceptions(SearchError)
     @wrap_search_exceptions()
-    def search_entry_last(self, query: str, entry: Tuple[str, str]) -> Tuple[Any, ...]:
+    def search_entry_last(self, query: str, entry: tuple[str, str]) -> tuple[Any, ...]:
         feed_url, entry_id = entry
 
         sql_query = (
@@ -863,9 +818,9 @@ class Search:
         now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
         sort: SearchSortOrder = 'relevant',
-        chunk_size: Optional[int] = None,
-        last: Optional[_T] = None,
-    ) -> Iterable[Tuple[EntrySearchResult, Optional[_T]]]:
+        chunk_size: int | None = None,
+        last: _T | None = None,
+    ) -> Iterable[tuple[EntrySearchResult, _T | None]]:
         sql_query, context = make_search_entries_query(filter_options, sort)
 
         random_mark = ''.join(
@@ -880,7 +835,6 @@ class Search:
             after_mark=after_mark,
             # 255 letters / 4.7 letters per word (average in English)
             snippet_tokens=54,
-            recent_threshold=now - self.storage.recent_threshold,
         )
 
         row_factory = partial(
@@ -933,7 +887,7 @@ class Search:
 
 def make_search_entries_query(
     filter_options: EntryFilterOptions, sort: SearchSortOrder
-) -> Tuple[Query, Dict[str, Any]]:
+) -> tuple[Query, dict[str, Any]]:
     search = (
         Query()
         .SELECT(
@@ -1004,7 +958,7 @@ def make_search_entries_query(
 
 
 def entry_search_result_factory(
-    t: Tuple[Any, ...], before_mark: str, after_mark: str
+    t: tuple[Any, ...], before_mark: str, after_mark: str
 ) -> EntrySearchResult:
     (
         entry_id,
@@ -1026,7 +980,7 @@ def entry_search_result_factory(
             '.feed.title' if not is_feed_user_title else '.feed.user_title'
         ] = HighlightedString.extract(feed_title, before_mark, after_mark)
 
-    rv_content: Dict[str, HighlightedString] = OrderedDict(
+    rv_content: dict[str, HighlightedString] = OrderedDict(
         (
             c['path'],
             HighlightedString.extract(c['value'], before_mark, after_mark),
