@@ -19,83 +19,19 @@ https://death.andgravity.com/own-query-builder
 https://death.andgravity.com/query-builder-how
     Code walk-through / tutorial.
     Low-level design decisions.
-    Better thought-out versions of the unimplemented features below.
-
----
-
-For a version of this that supports UNIONs see the second prototype in
-https://github.com/lemon24/reader/issues/123#issuecomment-624045621
-
-Here's a version based on the same idea that works with the 2021 refactor:
-https://gist.github.com/lemon24/3777dd39c96bf30014a331943ed789fa
-
----
-
-For a version of this that supports INSERT/UPDATE/DELETE, see
-https://github.com/lemon24/reader/commit/7c97fccf61d16946176c2455be3634c5a8343e1b
-
-The way things work now has changed slightly;
-we'd probably make them flag keywords now.
-
-To make VALUES bake in the parentheses, we just need to set:
-
-    query.formats[0]['VALUES'] = '({value})'
-
-That's to add one values tuple at a time. To add one column, we could do this:
-
-* output keyword even if called with no args
-* add(..., flag=...), and allow arbitrary flags;
-  INSERT_INTO('x', 'y', flag='table') -> flag is "INTO table"
-* parens_keywords = {'INSERT', 'VALUES'};
-  different than subquery_keywords because it applies once to the whole set
-
----
-
-To support marking arbitrary things as subqueries,
-add a signalling tuple and a helper function:
-
-    class _Subquery(tuple): pass
-
-    def Subquery(*args) -> _Subquery:
-        return _Subquery(args)  # from_arg() has to support 1-tuples
-
-Then, in from_arg(), if arg is a _Subquery, set is_subquery.
-
-Usage looks like this:
-
-    Query().FROM(
-        Subquery('alias', 'subquery'),
-        ('alias', 'not subquery'),
-    )
-
-Alternatively, add an is_subquery kwarg to add():
-
-    query = Query().FROM(('alias', 'subquery'), is_subquery=True)
-    query.FROM(('alias', 'not subquery'))
-
----
-
-To support using Queries as arguments directly,
-without having to convert them to strings first,
-allow _Thing.value to be a Query (and support it in);
-then, in _lines_keyword(), convert queries to str and override is_subquery.
+    How to implement: INSERT/UPDATE/DELETE, subqueries, UNION/...
 
 """
+from __future__ import annotations
+
 import functools
 import textwrap
 from collections import defaultdict
+from collections.abc import Iterable
 from typing import Any
 from typing import Callable
 from typing import cast
-from typing import Dict
-from typing import Iterable
-from typing import Iterator
-from typing import List
-from typing import Mapping
 from typing import NamedTuple
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
 from typing import TYPE_CHECKING
 from typing import TypeVar
 from typing import Union
@@ -105,7 +41,7 @@ _U = TypeVar('_U')
 
 
 _Q = TypeVar('_Q', bound='BaseQuery')
-_QArg = Union[str, Tuple[str, ...]]
+_QArg = Union[str, tuple[str, ...]]
 
 
 class _Thing(NamedTuple):
@@ -115,7 +51,7 @@ class _Thing(NamedTuple):
     is_subquery: bool = False
 
     @classmethod
-    def from_arg(cls, arg: _QArg, **kwargs: Any) -> '_Thing':
+    def from_arg(cls, arg: _QArg, **kwargs: Any) -> _Thing:
         if isinstance(arg, str):
             alias, value = '', arg
         elif len(arg) == 2:
@@ -125,7 +61,7 @@ class _Thing(NamedTuple):
         return cls(_clean_up(value), _clean_up(alias), **kwargs)
 
 
-class _FlagList(List[_T]):
+class _FlagList(list[_T]):
     flag: str = ''
 
 
@@ -146,10 +82,10 @@ class BaseQuery:
         'LIMIT',
     ]
 
-    separators: Mapping[str, str] = dict(WHERE='AND', HAVING='AND')
+    separators: dict[str, str] = dict(WHERE='AND', HAVING='AND')
     default_separator = ','
 
-    formats: Tuple[Mapping[str, str], ...] = (
+    formats: tuple[dict[str, str], ...] = (
         defaultdict(lambda: '{value}'),
         defaultdict(lambda: '{value} AS {alias}', WITH='{alias} AS {value}'),
     )
@@ -160,10 +96,10 @@ class BaseQuery:
 
     def __init__(
         self,
-        data: Optional[Mapping[str, Iterable[_QArg]]] = None,
-        separators: Optional[Mapping[str, str]] = None,
+        data: dict[str, Iterable[_QArg]] | None = None,
+        separators: dict[str, str] | None = None,
     ) -> None:
-        self.data: Mapping[str, _FlagList[_Thing]] = {}
+        self.data: dict[str, _FlagList[_Thing]] = {}
         if data is None:
             data = dict.fromkeys(self.keywords, ())
         for keyword, args in data.items():
@@ -183,7 +119,7 @@ class BaseQuery:
                 raise ValueError(f"{keyword} already has flag: {flag!r}")
             target.flag = flag
 
-        kwargs: Dict[str, Any] = {}
+        kwargs: dict[str, Any] = {}
         if fake_keyword:
             kwargs.update(keyword=fake_keyword)
         if keyword in self.subquery_keywords:
@@ -194,13 +130,13 @@ class BaseQuery:
 
         return self
 
-    def _resolve_fakes(self, keyword: str) -> Tuple[str, str]:
+    def _resolve_fakes(self, keyword: str) -> tuple[str, str]:
         for part, real in self.fake_keywords.items():
             if part in keyword:
                 return real, keyword
         return keyword, ''
 
-    def _resolve_flags(self, keyword: str) -> Tuple[str, str]:
+    def _resolve_flags(self, keyword: str) -> tuple[str, str]:
         prefix, _, flag = keyword.partition(' ')
         if prefix in self.flag_keywords:
             if flag and flag not in self.flag_keywords[prefix]:
@@ -227,13 +163,13 @@ class BaseQuery:
             else:
                 yield f'{keyword}\n'
 
-            grouped: Tuple[List[_Thing], ...] = ([], [])
+            grouped: tuple[list[_Thing], ...] = ([], [])
             for thing in things:
                 grouped[bool(thing.keyword)].append(thing)
             for group in grouped:
                 yield from self._lines_keyword(keyword, group)
 
-    def _lines_keyword(self, keyword: str, things: Sequence[_Thing]) -> Iterable[str]:
+    def _lines_keyword(self, keyword: str, things: list[_Thing]) -> Iterable[str]:
         for i, thing in enumerate(things):
             last = i + 1 == len(things)
 
@@ -280,11 +216,11 @@ class ScrollingWindowMixin(_SWMBase):
         order = 'DESC' if desc else 'ASC'
         return self.ORDER_BY(*(f'{thing} {order}' for thing in things))
 
-    def extract_last(self, result: Tuple[_T, ...]) -> Optional[Tuple[_T, ...]]:
+    def extract_last(self, result: tuple[_T, ...]) -> tuple[_T, ...] | None:
         names = [t.alias or t.value for t in self.data['SELECT']]
         return tuple(result[names.index(t)] for t in self.__things) or None
 
-    def add_last(self, last: Optional[Tuple[_T, ...]]) -> Sequence[Tuple[str, _T]]:
+    def add_last(self, last: tuple[_T, ...] | None) -> list[tuple[str, _T]]:
         self.__add_last()
         return self.__last_params(last)
 
@@ -296,7 +232,7 @@ class ScrollingWindowMixin(_SWMBase):
         comparison = BaseQuery({'(': self.__things, f') {op} (': labels, ')': ['']})
         self.add(self.__keyword, str(comparison).rstrip())
 
-    def __last_params(self, last: Optional[Tuple[_T, ...]]) -> Sequence[Tuple[str, _T]]:
+    def __last_params(self, last: tuple[_T, ...] | None) -> list[tuple[str, _T]]:
         return [(self.__make_label(i), t) for i, t in enumerate(last or ())]
 
 
@@ -309,13 +245,13 @@ if TYPE_CHECKING:  # pragma: no cover
 
 
 def paginated_query(
-    db: 'sqlite3.Connection',
+    db: sqlite3.Connection,
     query: Query,
-    params: Dict[str, Any] = {},  # noqa: B006
-    chunk_size: Optional[int] = 0,
-    last: Optional[_U] = None,
-    row_factory: Optional[Callable[[Tuple[Any, ...]], _T]] = None,
-) -> Iterator[Tuple[_T, _U]]:
+    params: dict[str, Any] = {},  # noqa: B006
+    chunk_size: int | None = 0,
+    last: _U | None = None,
+    row_factory: Callable[[tuple[Any, ...]], _T] | None = None,
+) -> Iterable[tuple[_T, _U]]:
 
     params = dict(params)
 
