@@ -1,21 +1,18 @@
+from __future__ import annotations
+
 import json
 import logging
 import sqlite3
+from collections.abc import Iterable
+from collections.abc import Mapping
+from collections.abc import Sequence
 from datetime import datetime
 from datetime import timedelta
 from functools import partial
 from typing import Any
-from typing import Dict
-from typing import Iterable
-from typing import Mapping
 from typing import NamedTuple
-from typing import Optional
 from typing import overload
-from typing import Sequence
-from typing import Tuple
-from typing import Type
 from typing import TypeVar
-from typing import Union
 
 from ._sql_utils import BaseQuery
 from ._sql_utils import paginated_query
@@ -132,15 +129,16 @@ def create_entries(db: sqlite3.Connection, name: str = 'entries') -> None:
             data_hash_changed INTEGER,  -- metadata about data_hash
 
             -- reader data
-            read INTEGER NOT NULL DEFAULT 0,
+            read INTEGER,
             read_modified TIMESTAMP,
-            important INTEGER NOT NULL DEFAULT 0,
+            important INTEGER,
             important_modified TIMESTAMP,
             added_by TEXT NOT NULL,
             last_updated TIMESTAMP NOT NULL,
             first_updated TIMESTAMP NOT NULL,
             first_updated_epoch TIMESTAMP NOT NULL,
             feed_order INTEGER NOT NULL,
+            recent_sort TIMESTAMP NOT NULL,
 
             PRIMARY KEY (id, feed),
             FOREIGN KEY (feed) REFERENCES feeds(url)
@@ -200,8 +198,8 @@ def create_entry_tags(db: sqlite3.Connection) -> None:
 
 class SchemaInfo(NamedTuple):
     table_prefix: str
-    id_columns: Tuple[str, ...]
-    not_found_exc: Type[ReaderError]
+    id_columns: tuple[str, ...]
+    not_found_exc: type[ReaderError]
 
 
 SCHEMA_INFO = {
@@ -212,35 +210,126 @@ SCHEMA_INFO = {
 
 
 def create_indexes(db: sqlite3.Connection) -> None:
+    create_entries_by_recent_index(db)
+    create_entries_by_feed_index(db)
+
+
+def create_entries_by_recent_index(db: sqlite3.Connection) -> None:
     # Speed up get_entries() queries that use apply_recent().
     db.execute(
         """
-        CREATE INDEX entries_by_kinda_first_updated ON entries(
-            first_updated_epoch,
-            coalesce(published, updated, first_updated),
-            feed,
-            last_updated,
-            - feed_order,
-            id
-        );
-        """
-    )
-    db.execute(
-        """
-        CREATE INDEX entries_by_kinda_published ON entries (
-            coalesce(published, updated, first_updated),
-            coalesce(published, updated, first_updated),
-            feed,
-            last_updated,
-            - feed_order,
-            id
+        CREATE INDEX entries_by_recent ON entries(
+            recent_sort DESC,
+            coalesce(published, updated, first_updated) DESC,
+            feed DESC,
+            last_updated DESC,
+            - feed_order DESC,
+            id DESC
         );
         """
     )
+
+
+def create_entries_by_feed_index(db: sqlite3.Connection) -> None:
     # Speed up get_entry_counts(feed=...).
     db.execute("CREATE INDEX entries_by_feed ON entries (feed);")
 
 
+def update_from_36_to_37(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # for https://github.com/lemon24/reader/issues/279
+    db.execute("ALTER TABLE entries ADD COLUMN recent_sort TIMESTAMP;")
+    db.execute(
+        """
+        UPDATE entries
+        SET recent_sort = coalesce(published, updated, first_updated_epoch);
+        """
+    )
+    db.execute("DROP INDEX entries_by_kinda_first_updated;")
+    db.execute("DROP INDEX entries_by_kinda_published;")
+    create_entries_by_recent_index(db)
+
+
+def recreate_search_triggers(db: sqlite3.Connection) -> None:  # pragma: no cover
+    from ._search import Search
+
+    if Search._is_enabled(db):
+        Search._drop_triggers(db)
+        Search._create_triggers(db)
+
+
+def update_from_37_to_38(db: sqlite3.Connection) -> None:  # pragma: no cover
+    # https://github.com/lemon24/reader/issues/254#issuecomment-1404215814
+
+    create_entries(db, 'new_entries')
+    db.execute(
+        """
+        INSERT INTO new_entries (
+            id,
+            feed,
+            title,
+            link,
+            updated,
+            author,
+            published,
+            summary,
+            content,
+            enclosures,
+            original_feed,
+            data_hash,
+            data_hash_changed,
+            read,
+            read_modified,
+            important,
+            important_modified,
+            added_by,
+            last_updated,
+            first_updated,
+            first_updated_epoch,
+            feed_order,
+            recent_sort
+        )
+        SELECT
+            id,
+            feed,
+            title,
+            link,
+            updated,
+            author,
+            published,
+            summary,
+            content,
+            enclosures,
+            original_feed,
+            data_hash,
+            data_hash_changed,
+            read,
+            read_modified,
+            CASE
+                WHEN read AND NOT important AND important_modified is not NULL
+                    THEN 0
+                WHEN NOT important
+                    THEN NULL
+                ELSE important
+            END,
+            important_modified,
+            added_by,
+            last_updated,
+            first_updated,
+            first_updated_epoch,
+            feed_order,
+            recent_sort
+        FROM entries;
+        """
+    )
+
+    # IMPORTANT: this drops ALL indexes and triggers ON entries
+    db.execute("DROP TABLE entries;")
+    db.execute("ALTER TABLE new_entries RENAME TO entries;")
+
+    create_indexes(db)
+    recreate_search_triggers(db)
+
+
 # Row value support was added in 3.15.
 # TODO: Remove the Search.update() check once this gets bumped to >=3.18.
 MINIMUM_SQLITE_VERSION = (3, 15)
@@ -248,16 +337,18 @@ MINIMUM_SQLITE_VERSION = (3, 15)
 REQUIRED_SQLITE_FUNCTIONS = ['json_array_length']
 
 
-def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
+def setup_db(db: sqlite3.Connection, wal_enabled: bool | None) -> None:
     return setup_sqlite_db(
         db,
         create=create_db,
-        version=36,
+        version=38,
         migrations={
             # 1-9 removed before 0.1 (last in e4769d8ba77c61ec1fe2fbe99839e1826c17ace7)
             # 10-16 removed before 1.0 (last in 618f158ebc0034eefb724a55a84937d21c93c1a7)
             # 17-28 removed before 2.0 (last in be9c89581ea491d0c9cc95c9d39f073168a2fd02)
             # 29-35 removed before 3.0 (last in 69c75529a3f80107b68346d592d6450f9725187c)
+            36: update_from_36_to_37,
+            37: update_from_37_to_38,
         },
         id=APPLICATION_ID,
         minimum_sqlite_version=MINIMUM_SQLITE_VERSION,
@@ -289,17 +380,16 @@ def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
 
 MISSING_MIGRATION_DETAIL = (
     "; you may have skipped some required migrations, see "
-    "https://reader.readthedocs.io/en/latest/changelog.html#removed-migrations-2-0"
+    "https://reader.readthedocs.io/en/latest/changelog.html#removed-migrations-3-0"
 )
 
 
 class Storage:
 
-    # recent_threshold, chunk_size, and entry_counts_average_periods
+    # chunk_size and entry_counts_average_periods
     # are not part of the Storage interface,
     # but are part of the private API of this implementation.
-    recent_threshold = timedelta(7)
-    chunk_size = 2 ** 8
+    chunk_size = 2**8
     # 1, 3, 12 months rounded down to days,
     # assuming an average of 30.436875 days/month
     entry_counts_average_periods = (30, 91, 365)
@@ -308,11 +398,11 @@ class Storage:
     def __init__(
         self,
         path: str,
-        timeout: Optional[float] = None,
-        wal_enabled: Optional[bool] = True,
-        factory: Optional[Type[sqlite3.Connection]] = None,
+        timeout: float | None = None,
+        wal_enabled: bool | None = True,
+        factory: type[sqlite3.Connection] | None = None,
     ):
-        kwargs: Dict[str, Any] = {}
+        kwargs: dict[str, Any] = {}
         if timeout is not None:
             kwargs['timeout'] = timeout
         if factory:  # pragma: no cover
@@ -438,8 +528,8 @@ class Storage:
         self,
         filter_options: FeedFilterOptions = FeedFilterOptions(),  # noqa: B008
         sort: FeedSortOrder = 'title',
-        limit: Optional[int] = None,
-        starting_after: Optional[str] = None,
+        limit: int | None = None,
+        starting_after: str | None = None,
     ) -> Iterable[Feed]:
         rv = join_paginated_iter(
             partial(self.get_feeds_page, filter_options, sort),  # type: ignore[arg-type]
@@ -450,7 +540,7 @@ class Storage:
         yield from rv
 
     @wrap_exceptions(StorageError)
-    def get_feed_last(self, sort: str, url: str) -> Tuple[Any, ...]:
+    def get_feed_last(self, sort: str, url: str) -> tuple[Any, ...]:
         # TODO: make this method private?
 
         query = Query().FROM("feeds").WHERE("url = :url")
@@ -473,9 +563,9 @@ class Storage:
         self,
         filter_options: FeedFilterOptions = FeedFilterOptions(),  # noqa: B008
         sort: FeedSortOrder = 'title',
-        chunk_size: Optional[int] = None,
-        last: Optional[_T] = None,
-    ) -> Iterable[Tuple[Feed, Optional[_T]]]:
+        chunk_size: int | None = None,
+        last: _T | None = None,
+    ) -> Iterable[tuple[Feed, _T | None]]:
         query, context = make_get_feeds_query(filter_options, sort)
         yield from paginated_query(
             self.get_db(), query, context, chunk_size, last, feed_factory
@@ -511,8 +601,8 @@ class Storage:
         # so we don't expose any pagination stuff.
 
         def inner(
-            chunk_size: Optional[int], last: Optional[_T]
-        ) -> Iterable[Tuple[FeedForUpdate, Optional[_T]]]:
+            chunk_size: int | None, last: _T | None
+        ) -> Iterable[tuple[FeedForUpdate, _T | None]]:
             query = (
                 Query()
                 .SELECT(
@@ -541,8 +631,8 @@ class Storage:
         yield from join_paginated_iter(inner, self.chunk_size)
 
     def _get_entries_for_update(
-        self, entries: Iterable[Tuple[str, str]]
-    ) -> Iterable[Optional[EntryForUpdate]]:
+        self, entries: Iterable[tuple[str, str]]
+    ) -> Iterable[EntryForUpdate | None]:
         # We could fetch everything in a single query,
         # but there are limits to the number of variables used in a query,
         # and we'd have to fall back to this anyway.
@@ -581,8 +671,8 @@ class Storage:
 
     @wrap_exceptions_iter(StorageError)
     def get_entries_for_update(
-        self, entries: Iterable[Tuple[str, str]]
-    ) -> Iterable[Optional[EntryForUpdate]]:
+        self, entries: Iterable[tuple[str, str]]
+    ) -> Iterable[EntryForUpdate | None]:
         # It's acceptable for this method to not be atomic. TODO: Why?
 
         iterables = chunks(self.chunk_size, entries) if self.chunk_size else (entries,)
@@ -594,7 +684,7 @@ class Storage:
             yield from rv
 
     @wrap_exceptions(StorageError)
-    def set_feed_user_title(self, url: str, title: Optional[str]) -> None:
+    def set_feed_user_title(self, url: str, title: str | None) -> None:
         with self.get_db() as db:
             cursor = db.execute(
                 "UPDATE feeds SET user_title = :title WHERE url = :url;",
@@ -621,7 +711,7 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def mark_as_read(
-        self, feed_url: str, entry_id: str, read: bool, modified: Optional[datetime]
+        self, feed_url: str, entry_id: str, read: bool, modified: datetime | None
     ) -> None:
         with self.get_db() as db:
             cursor = db.execute(
@@ -643,8 +733,8 @@ class Storage:
         self,
         feed_url: str,
         entry_id: str,
-        important: bool,
-        modified: Optional[datetime],
+        important: bool | None,
+        modified: datetime | None,
     ) -> None:
         with self.get_db() as db:
             cursor = db.execute(
@@ -664,6 +754,40 @@ class Storage:
             )
         rowcount_exactly_one(cursor, lambda: EntryNotFoundError(feed_url, entry_id))
 
+    # TODO: (maybe) unified pair of methods to get/set "hidden" entry attributes
+
+    @wrap_exceptions(StorageError)
+    def get_entry_recent_sort(self, entry: tuple[str, str]) -> datetime:
+        feed_url, entry_id = entry
+        rows = self.get_db().execute(
+            """
+            SELECT recent_sort
+            FROM entries
+            WHERE feed = :feed_url AND id = :entry_id;
+            """,
+            dict(feed_url=feed_url, entry_id=entry_id),
+        )
+        return zero_or_one(
+            (r[0] for r in rows), lambda: EntryNotFoundError(feed_url, entry_id)
+        )
+
+    @wrap_exceptions(StorageError)
+    def set_entry_recent_sort(
+        self, entry: tuple[str, str], recent_sort: datetime
+    ) -> None:
+        feed_url, entry_id = entry
+        with self.get_db() as db:
+            cursor = db.execute(
+                """
+                UPDATE entries
+                SET
+                    recent_sort = :recent_sort
+                WHERE feed = :feed_url AND id = :entry_id;
+                """,
+                dict(feed_url=feed_url, entry_id=entry_id, recent_sort=recent_sort),
+            )
+        rowcount_exactly_one(cursor, lambda: EntryNotFoundError(feed_url, entry_id))
+
     @wrap_exceptions(StorageError)
     def update_feed(self, intent: FeedUpdateIntent) -> None:
         url, last_updated, feed, http_etag, http_last_modified, last_exception = intent
@@ -792,6 +916,7 @@ class Storage:
                 first_updated,
                 first_updated_epoch,
                 feed_order,
+                recent_sort,
                 original_feed,
                 data_hash,
                 data_hash_changed,
@@ -817,11 +942,11 @@ class Storage:
                     FROM entries
                     WHERE id = :id AND feed = :feed_url
                 ),
-                coalesce((
+                (
                     SELECT important
                     FROM entries
                     WHERE id = :id AND feed = :feed_url
-                ), 0),
+                ),
                 (
                     SELECT important_modified
                     FROM entries
@@ -839,6 +964,11 @@ class Storage:
                     WHERE id = :id AND feed = :feed_url
                 )),
                 :feed_order,
+                coalesce(:recent_sort, (
+                    SELECT recent_sort
+                    FROM entries
+                    WHERE id = :id AND feed = :feed_url
+                )),
                 NULL, -- original_feed
                 :data_hash,
                 :data_hash_changed,
@@ -883,7 +1013,7 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def delete_entries(
-        self, entries: Iterable[Tuple[str, str]], *, added_by: Optional[str] = None
+        self, entries: Iterable[tuple[str, str]], *, added_by: str | None = None
     ) -> None:
         # This must be atomic (unlike add_or_update_entries()); hence, no paging.
         # We'll deal with locking issues only if they start appearing
@@ -911,7 +1041,7 @@ class Storage:
 
                 cursor.execute(delete_query, context)
                 rowcount_exactly_one(
-                    cursor, lambda: EntryNotFoundError(feed_url, entry_id)
+                    cursor, lambda: EntryNotFoundError(feed_url, entry_id)  # noqa: B023
                 )
 
     def get_entries(
@@ -919,8 +1049,8 @@ class Storage:
         now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
         sort: EntrySortOrder = 'recent',
-        limit: Optional[int] = None,
-        starting_after: Optional[Tuple[str, str]] = None,
+        limit: int | None = None,
+        starting_after: tuple[str, str] | None = None,
     ) -> Iterable[Entry]:
         # TODO: deduplicate
         if sort == 'recent':
@@ -948,8 +1078,8 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def get_entry_last(
-        self, now: datetime, sort: str, entry: Tuple[str, str]
-    ) -> Tuple[Any, ...]:
+        self, now: datetime, sort: str, entry: tuple[str, str]
+    ) -> tuple[Any, ...]:
         # TODO: make this method private?
 
         feed_url, entry_id = entry
@@ -964,12 +1094,7 @@ class Storage:
         else:
             assert False, "shouldn't get here"  # noqa: B011; # pragma: no cover
 
-        context = dict(
-            feed=feed_url,
-            id=entry_id,
-            # TODO: duplicated from get_entries_page()
-            recent_threshold=now - self.recent_threshold,
-        )
+        context = dict(feed=feed_url, id=entry_id)
 
         return zero_or_one(
             self.get_db().execute(str(query), context),
@@ -982,11 +1107,10 @@ class Storage:
         now: datetime,
         filter_options: EntryFilterOptions = EntryFilterOptions(),  # noqa: B008
         sort: EntrySortOrder = 'recent',
-        chunk_size: Optional[int] = None,
-        last: Optional[_T] = None,
-    ) -> Iterable[Tuple[Entry, Optional[_T]]]:
+        chunk_size: int | None = None,
+        last: _T | None = None,
+    ) -> Iterable[tuple[Entry, _T | None]]:
         query, context = make_get_entries_query(filter_options, sort)
-        context.update(recent_threshold=now - self.recent_threshold)
         yield from paginated_query(
             self.get_db(), query, context, chunk_size, last, entry_factory
         )
@@ -1013,8 +1137,8 @@ class Storage:
     def get_tags(
         self,
         resource_id: AnyResourceId,
-        key: Optional[str] = None,
-    ) -> Iterable[Tuple[str, JSONType]]:
+        key: str | None = None,
+    ) -> Iterable[tuple[str, JSONType]]:
         yield from join_paginated_iter(
             partial(self.get_tags_page, resource_id, key),
             self.chunk_size,
@@ -1024,12 +1148,12 @@ class Storage:
     def get_tags_page(
         self,
         resource_id: AnyResourceId,
-        key: Optional[str] = None,
-        chunk_size: Optional[int] = None,
-        last: Optional[_T] = None,
-    ) -> Iterable[Tuple[Tuple[str, JSONType], Optional[_T]]]:
+        key: str | None = None,
+        chunk_size: int | None = None,
+        last: _T | None = None,
+    ) -> Iterable[tuple[tuple[str, JSONType], _T | None]]:
         query = Query().SELECT("key")
-        context: Dict[str, Any] = dict()
+        context: dict[str, Any] = dict()
 
         if resource_id is not None:
             info = SCHEMA_INFO[len(resource_id)]
@@ -1057,7 +1181,7 @@ class Storage:
 
         query.scrolling_window_order_by("key")
 
-        def row_factory(t: Tuple[Any, ...]) -> Tuple[str, JSONType]:
+        def row_factory(t: tuple[Any, ...]) -> tuple[str, JSONType]:
             key, value, *_ = t
             return key, json.loads(value)
 
@@ -1080,7 +1204,7 @@ class Storage:
         self,
         resource_id: ResourceId,
         key: str,
-        value: Union[MissingType, JSONType] = MISSING,
+        value: MissingType | JSONType = MISSING,
     ) -> None:
         info = SCHEMA_INFO[len(resource_id)]
 
@@ -1144,7 +1268,7 @@ class Storage:
 
 def make_get_feeds_query(
     filter_options: FeedFilterOptions, sort: FeedSortOrder
-) -> Tuple[Query, Dict[str, Any]]:
+) -> tuple[Query, dict[str, Any]]:
     query = (
         Query()
         .SELECT(
@@ -1182,7 +1306,7 @@ def make_get_feeds_query(
     return query, context
 
 
-def feed_factory(t: Tuple[Any, ...]) -> Feed:
+def feed_factory(t: tuple[Any, ...]) -> Feed:
     return Feed._make(
         t[:10]
         + (
@@ -1195,10 +1319,10 @@ def feed_factory(t: Tuple[Any, ...]) -> Feed:
 def apply_feed_filter_options(
     query: Query,
     filter_options: FeedFilterOptions,
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     url, tags, broken, updates_enabled, new = filter_options
 
-    context: Dict[str, object] = {}
+    context: dict[str, object] = {}
 
     if url:
         query.WHERE("url = :url")
@@ -1219,7 +1343,7 @@ def apply_feed_filter_options(
 def make_get_entries_query(
     filter_options: EntryFilterOptions,
     sort: EntrySortOrder,
-) -> Tuple[Query, Dict[str, Any]]:
+) -> tuple[Query, dict[str, Any]]:
     query = (
         Query()
         .SELECT(
@@ -1275,14 +1399,14 @@ def make_get_entries_query(
     return query, filter_context
 
 
-def entry_factory(t: Tuple[Any, ...]) -> Entry:
+def entry_factory(t: tuple[Any, ...]) -> Entry:
     feed = feed_factory(t[0:12])
     entry = t[12:19] + (
         tuple(Content(**d) for d in json.loads(t[19])) if t[19] else (),
         tuple(Enclosure(**d) for d in json.loads(t[20])) if t[20] else (),
         t[21] == 1,
         t[22],
-        t[23] == 1,
+        t[23] == 1 if t[23] is not None else None,
         t[24],
         t[25],
         t[26],
@@ -1293,9 +1417,19 @@ def entry_factory(t: Tuple[Any, ...]) -> Entry:
     return Entry._make(entry)
 
 
+TRISTATE_FILTER_TO_SQL = dict(
+    istrue="({expr} IS NOT NULL AND {expr})",
+    isfalse="({expr} IS NOT NULL AND NOT {expr})",
+    notset="{expr} IS NULL",
+    nottrue="({expr} IS NULL OR NOT {expr})",
+    notfalse="({expr} IS NULL OR {expr})",
+    isset="{expr} IS NOT NULL",
+)
+
+
 def apply_entry_filter_options(
     query: Query, filter_options: EntryFilterOptions, keyword: str = 'WHERE'
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     add = getattr(query, keyword)
     feed_url, entry_id, read, important, has_enclosures, feed_tags = filter_options
 
@@ -1311,8 +1445,8 @@ def apply_entry_filter_options(
     if read is not None:
         add(f"{'' if read else 'NOT'} entries.read")
 
-    if important is not None:
-        add(f"{'' if important else 'NOT'} entries.important")
+    if important != 'any':
+        add(TRISTATE_FILTER_TO_SQL[important].format(expr='entries.important'))
 
     if has_enclosures is not None:
         add(
@@ -1337,7 +1471,7 @@ def apply_feed_tags_filter_options(
     tags: TagFilter,
     url_column: str,
     keyword: str = 'WHERE',
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     add = getattr(query, keyword)
 
     context = {}
@@ -1382,20 +1516,7 @@ def apply_feed_tags_filter_options(
 
 def make_recent_last_select(id_prefix: str = 'entries.') -> Sequence[Any]:
     return [
-        (
-            'kinda_first_updated',
-            """
-            coalesce (
-                CASE
-                WHEN
-                    coalesce(entries.published, entries.updated, entries.first_updated)
-                        >= :recent_threshold
-                    THEN entries.first_updated_epoch
-                END,
-                entries.published, entries.updated, entries.first_updated
-            )
-            """,
-        ),
+        'recent_sort',
         ('kinda_published', 'coalesce(published, updated, first_updated)'),
         f'{id_prefix}feed',
         'last_updated',
@@ -1409,69 +1530,51 @@ def apply_recent(
 ) -> None:
     """Change query to sort entries by "recent"."""
 
-    # Version of apply_recent() that takes advantage of indexes, implemented in
-    # <https://github.com/lemon24/reader/issues/134#issuecomment-722454963>.
-    # An older version that does not can be found in reader 1.9.
-    #
-    # WARNING: Always keep the entries_by_kinda_* indexes in sync with the ORDER BY of the by_kinda_* CTEs below.
+    # WARNING: Always keep the entries_by_recent index in sync
+    # with the ORDER BY of the CTE below.
 
-    def make_by_kinda_cte(recent: bool) -> str:
-        if recent:
-            kinda_first_updated = 'first_updated_epoch'
-            sign = '>='
-        else:
-            kinda_first_updated = 'coalesce(published, updated, first_updated)'
-            sign = '<'
-
-        return f"""
+    query.WITH(
+        (
+            'ids',
+            """
             SELECT
                 feed,
                 id,
                 last_updated,
-                {kinda_first_updated} as kinda_first_updated,
+                recent_sort,
                 coalesce(published, updated, first_updated) as kinda_published,
                 - feed_order as negative_feed_order
             FROM entries
-            WHERE kinda_published {sign} :recent_threshold
             ORDER BY
-                kinda_first_updated DESC,
+                recent_sort DESC,
                 kinda_published DESC,
                 feed DESC,
                 last_updated DESC,
                 negative_feed_order DESC,
                 id DESC
-        """
-
-    query.WITH(
-        ('by_kinda_first_updated', make_by_kinda_cte(recent=True)),
-        ('by_kinda_published', make_by_kinda_cte(recent=False)),
-        (
-            'ids',
-            """
-            SELECT * FROM by_kinda_first_updated
-            UNION ALL
-            SELECT * FROM by_kinda_published
             """,
         ),
     )
     query.JOIN(f"ids ON (ids.id, ids.feed) = ({id_prefix}id, {id_prefix}feed)")
 
     query.SELECT(
-        "ids.last_updated",
-        "kinda_first_updated",
-        "kinda_published",
-        "negative_feed_order",
+        'ids.recent_sort',
+        'ids.kinda_published',
+        'ids.feed',
+        'ids.last_updated',
+        'ids.negative_feed_order',
+        'ids.id',
     )
 
     # NOTE: when changing, ensure none of the values can be null
     # to prevent https://github.com/lemon24/reader/issues/203
     query.scrolling_window_order_by(
-        'kinda_first_updated',
-        'kinda_published',
-        f'{id_prefix}feed',
+        'ids.recent_sort',
+        'ids.kinda_published',
+        'ids.feed',
         'ids.last_updated',
-        'negative_feed_order',
-        f'{id_prefix}id',
+        'ids.negative_feed_order',
+        'ids.id',
         desc=True,
         keyword=keyword,
     )
@@ -1490,9 +1593,9 @@ def apply_random(query: Query) -> None:
 
 def make_entry_counts_query(
     now: datetime,
-    average_periods: Tuple[float, ...],
+    average_periods: tuple[float, ...],
     entries_query: Query,
-) -> Tuple[Query, Dict[str, Any]]:
+) -> tuple[Query, dict[str, Any]]:
     query = (
         Query()
         .WITH(('entries_filtered', str(entries_query)))
@@ -1517,7 +1620,7 @@ def make_entry_counts_query(
     # one CTE / period + HAVING in the CTE is a tiny bit faster than
     # one CTE + WHERE in the SELECT
 
-    context: Dict[str, Any] = dict(now=now)
+    context: dict[str, Any] = dict(now=now)
 
     for period_i, period_days in enumerate(average_periods):
         # TODO: when we get first_updated, use it instead of first_updated_epoch
