@@ -1,3 +1,8 @@
+from __future__ import annotations
+
+from collections.abc import Iterable
+from collections.abc import Mapping
+from collections.abc import Sequence
 from dataclasses import dataclass
 from datetime import datetime
 from datetime import timezone
@@ -5,13 +10,9 @@ from functools import cached_property
 from types import MappingProxyType
 from types import SimpleNamespace
 from typing import Any
-from typing import Iterable
-from typing import Mapping
+from typing import get_args
+from typing import Literal
 from typing import NamedTuple
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
-from typing import Type
 from typing import TypeVar
 from typing import Union
 
@@ -28,6 +29,7 @@ from .types import ExceptionInfo
 from .types import Feed
 from .types import FeedInput
 from .types import TagFilterInput
+from .types import TristateFilterInput
 
 # Private API
 # https://github.com/lemon24/reader/issues/111
@@ -44,27 +46,32 @@ class FeedData(_namedtuple_compat):
 
     """Feed data that comes from the feed.
 
-    Attributes are a subset of those of Feed.
+    Attributes are a subset of those of :class:`~reader.Feed`.
 
     """
 
     url: str
-    updated: Optional[datetime] = None
-    title: Optional[str] = None
-    link: Optional[str] = None
-    author: Optional[str] = None
-    subtitle: Optional[str] = None
-    version: Optional[str] = None
+    updated: datetime | None = None
+    title: str | None = None
+    link: str | None = None
+    author: str | None = None
+    subtitle: str | None = None
+    version: str | None = None
 
     def as_feed(self, **kwargs: object) -> Feed:
-        """For testing."""
+        """Convert this to a feed; kwargs override attributes.
+
+        Returns:
+            :class:`~reader.Feed`.
+
+        """
         attrs = dict(self.__dict__)
         attrs.pop('hash', None)
         attrs.update(kwargs)
         return Feed(**attrs)
 
     @property
-    def resource_id(self) -> Tuple[str]:
+    def resource_id(self) -> tuple[str]:
         return (self.url,)
 
     _hash_exclude_ = frozenset({'url', 'updated'})
@@ -79,39 +86,41 @@ class EntryData(_namedtuple_compat):
 
     """Entry data that comes from the feed.
 
-    Attributes are a subset of those of Entry.
-
-    ---
-
-    This is not generic anymore, as of 2.5, and will likely never be.
-
-    TODO: Make Entry a subclass of EntryData, make Feed a subclass of FeedData.
-
-    It may still not be possible to use it as a subclass, though, because:
-
-    * help(Entry) may not work
-    * Sphinx/autodoc may not work: https://github.com/sphinx-doc/sphinx/issues/741 (closed)
-    * as_entry(), hash() must not be inherited
+    Attributes are a subset of those of :class:`~reader.Entry`.
 
     """
 
-    #: The feed URL.
+    # This is not generic anymore, as of 2.5, and will likely never be.
+    #
+    # TODO: Make Entry a subclass of EntryData, make Feed a subclass of FeedData.
+    #
+    # It may still not be possible to use it as a subclass, though, because:
+    #
+    # * help(Entry) may not work
+    # * Sphinx/autodoc may not work: https://github.com/sphinx-doc/sphinx/issues/741 (closed)
+    # * as_entry(), hash() must not be inherited
+
     feed_url: str
 
     # WARNING: When changing attributes, keep Entry, EntryData, and entry_data_from_obj in sync.
 
     id: str
-    updated: Optional[datetime] = None
-    title: Optional[str] = None
-    link: Optional[str] = None
-    author: Optional[str] = None
-    published: Optional[datetime] = None
-    summary: Optional[str] = None
+    updated: datetime | None = None
+    title: str | None = None
+    link: str | None = None
+    author: str | None = None
+    published: datetime | None = None
+    summary: str | None = None
     content: Sequence[Content] = ()
     enclosures: Sequence[Enclosure] = ()
 
     def as_entry(self, **kwargs: object) -> Entry:
-        """For testing."""
+        """Convert this to an entry; kwargs override attributes.
+
+        Returns:
+            :class:`~reader.Entry`.
+
+        """
         attrs = dict(self.__dict__)
         feed_url = attrs.pop('feed_url')
         attrs.pop('hash', None)
@@ -121,7 +130,7 @@ class EntryData(_namedtuple_compat):
         return Entry(**attrs)
 
     @property
-    def resource_id(self) -> Tuple[str, str]:
+    def resource_id(self) -> tuple[str, str]:
         return self.feed_url, self.id
 
     _hash_exclude_ = frozenset({'feed_url', 'id', 'updated'})
@@ -174,7 +183,7 @@ def enclosure_from_obj(obj: object) -> Enclosure:
     )
 
 
-def _getattr(obj: object, name: str, type: Type[_T]) -> _T:
+def _getattr(obj: object, name: str, type: type[_T]) -> _T:
     # will raise AttributeError implicitly
     value = getattr(obj, name)
     if not isinstance(value, type):
@@ -184,7 +193,7 @@ def _getattr(obj: object, name: str, type: Type[_T]) -> _T:
     return value
 
 
-def _getattr_optional(obj: object, name: str, type: Type[_T]) -> Optional[_T]:
+def _getattr_optional(obj: object, name: str, type: type[_T]) -> _T | None:
     value = getattr(obj, name, None)
     if value is None:
         return value
@@ -195,7 +204,7 @@ def _getattr_optional(obj: object, name: str, type: Type[_T]) -> Optional[_T]:
     return value
 
 
-def _getattr_optional_datetime(obj: object, name: str) -> Optional[datetime]:
+def _getattr_optional_datetime(obj: object, name: str) -> datetime | None:
     value = _getattr_optional(obj, name, datetime)
     if value is None:
         return value
@@ -203,39 +212,53 @@ def _getattr_optional_datetime(obj: object, name: str) -> Optional[datetime]:
 
 
 class ParsedFeed(NamedTuple):
+    """A parsed feed."""
 
+    #: The feed.
     feed: FeedData
     # TODO: wrap entries in iter(entries) to ensure stuff doesn't rely on it being a list
+    #: Iterable of entries.
     entries: Iterable[EntryData]
-    http_etag: Optional[str] = None
-    http_last_modified: Optional[str] = None
-    mime_type: Optional[str] = None
+    #: The HTTP ``ETag`` header associated with the feed resource.
+    #: Passed back to the retriever on the next update.
+    http_etag: str | None = None
+    #: The HTTP ``Last-Modified`` header associated with the feed resource.
+    #: Passed back to the retriever on the next update.
+    http_last_modified: str | None = None
+    #: The MIME type of the feed resource.
+    #: Used by :meth:`~reader._parser.Parser.process_entry_pairs`
+    #: to select an appropriate parser.
+    mime_type: str | None = None
 
 
 class FeedForUpdate(NamedTuple):
 
-    """Update-relevant information about an exiting feed, from Storage."""
+    """Update-relevant information about an existing feed, from Storage."""
 
+    #: The feed URL.
     url: str
 
     #: The date the feed was last updated, according to the feed.
-    updated: Optional[datetime]
+    updated: datetime | None
 
-    http_etag: Optional[str]
-    http_last_modified: Optional[str]
+    #: The HTTP ``ETag`` header from the last update.
+    http_etag: str | None
+
+    #: The HTTP ``Last-Modified`` header from the last update.
+    http_last_modified: str | None
 
     #: Whether the next update should update *all* entries,
     #: regardless of their .updated.
     stale: bool
 
     #: The date the feed was last updated, according to reader; none if never.
-    last_updated: Optional[datetime]
+    last_updated: datetime | None
 
     #: Whether the feed had an exception at the last update.
     last_exception: bool
 
-    #: The hash of the corresponding FeedData.
-    hash: Optional[bytes]
+    #: The :attr:`~FeedData.hash` of the corresponding FeedData.
+    hash: bytes | None
 
 
 class EntryForUpdate(NamedTuple):
@@ -243,17 +266,17 @@ class EntryForUpdate(NamedTuple):
     """Update-relevant information about an existing entry, from Storage."""
 
     #: The date the entry was last updated, according to the entry.
-    updated: Optional[datetime]
+    updated: datetime | None
 
     #: The date the entry was published, according to the entry.
-    published: Optional[datetime]
+    published: datetime | None
 
-    #: The hash of the corresponding EntryData.
-    hash: Optional[bytes]
+    #: The :attr:`~EntryData.hash` of the corresponding EntryData.
+    hash: bytes | None
 
     #: The number of updates due to a different ``hash``
     #: since the last time ``updated`` changed.
-    hash_changed: Optional[int]
+    hash_changed: int | None
 
 
 class FeedUpdateIntent(NamedTuple):
@@ -263,16 +286,16 @@ class FeedUpdateIntent(NamedTuple):
     url: str
 
     #: The time at the start of updating this feed.
-    last_updated: Optional[datetime]
+    last_updated: datetime | None
 
-    feed: Optional[FeedData] = None
-    http_etag: Optional[str] = None
-    http_last_modified: Optional[str] = None
+    feed: FeedData | None = None
+    http_etag: str | None = None
+    http_last_modified: str | None = None
 
     # TODO: Is there a better way of modeling/enforcing these? A sort of tagged union, maybe? (last_updated should be non-optional then)
 
     #: Cause of ParseError, if any; if set, everything else except url should be None.
-    last_exception: Optional[ExceptionInfo] = None
+    last_exception: ExceptionInfo | None = None
 
 
 class EntryUpdateIntent(NamedTuple):
@@ -288,18 +311,24 @@ class EntryUpdateIntent(NamedTuple):
 
     #: First last_updated (sets Entry.added).
     #: None if the entry already exists.
-    first_updated: Optional[datetime]
+    first_updated: datetime | None
 
     #: The time at the start of updating this batch of feeds (start of
     #: update_feed in update_feed, start of update_feeds in update_feeds);
     #: None if the entry already exists.
-    first_updated_epoch: Optional[datetime]
+    first_updated_epoch: datetime | None
+
+    #: Sort key for get_entries(sort='recent').
+    #: Currently, first_updated_epoch if the feed is not new,
+    #: published or updated or first_updated_epoch otherwise.
+    #: None if the entry already exists.
+    recent_sort: datetime | None
 
     #: The index of the entry in the feed (zero-based).
     feed_order: int = 0
 
     #: Same as EntryForUpdate.hash_changed.
-    hash_changed: Optional[int] = 0
+    hash_changed: int | None = 0
 
     #: Same as Entry.source.
     added_by: EntryAddedBy = 'feed'
@@ -313,7 +342,7 @@ class EntryUpdateIntent(NamedTuple):
 # TODO: these should probably be in storage.py (along with some of the above)
 
 
-TagFilter = Sequence[Sequence[Union[bool, Tuple[bool, str]]]]
+TagFilter = Sequence[Sequence[Union[bool, tuple[bool, str]]]]
 
 
 def tag_filter_argument(tags: TagFilterInput, name: str = 'tags') -> TagFilter:
@@ -324,7 +353,7 @@ def tag_filter_argument(tags: TagFilterInput, name: str = 'tags') -> TagFilter:
     if not isinstance(tags, Sequence) or isinstance(tags, str):
         raise ValueError(f"{name} must be none, bool, or a non-string sequence")
 
-    def normalize_tag(tag: Union[str, bool]) -> Union[bool, Tuple[bool, str]]:
+    def normalize_tag(tag: str | bool) -> bool | tuple[bool, str]:
         if isinstance(tag, bool):
             return tag
 
@@ -362,6 +391,31 @@ def tag_filter_argument(tags: TagFilterInput, name: str = 'tags') -> TagFilter:
     return rv
 
 
+TristateFilter = Literal[
+    'istrue',
+    'isfalse',
+    'notset',
+    'nottrue',
+    'notfalse',
+    'isset',
+    'any',
+]
+
+
+def tristate_filter_argument(value: TristateFilterInput, name: str) -> TristateFilter:
+    # https://github.com/lemon24/reader/issues/254#issuecomment-1435648359
+    if value is None:
+        return 'any'
+    if value == True:  # noqa: E712
+        return 'istrue'
+    if value == False:  # noqa: E712
+        return 'nottrue'
+    args = get_args(TristateFilter)
+    if value in args:
+        return value
+    raise ValueError(f"{name} must be none, bool, or one of {args}")
+
+
 _EFO = TypeVar('_EFO', bound='EntryFilterOptions')
 
 
@@ -369,21 +423,21 @@ class EntryFilterOptions(NamedTuple):
 
     """Options for filtering the results of the "get entry" storage methods."""
 
-    feed_url: Optional[str] = None
-    entry_id: Optional[str] = None
-    read: Optional[bool] = None
-    important: Optional[bool] = None
-    has_enclosures: Optional[bool] = None
+    feed_url: str | None = None
+    entry_id: str | None = None
+    read: bool | None = None
+    important: TristateFilter = 'any'
+    has_enclosures: bool | None = None
     feed_tags: TagFilter = ()
 
     @classmethod
     def from_args(
-        cls: Type[_EFO],
-        feed: Optional[FeedInput] = None,
-        entry: Optional[EntryInput] = None,
-        read: Optional[bool] = None,
-        important: Optional[bool] = None,
-        has_enclosures: Optional[bool] = None,
+        cls: type[_EFO],
+        feed: FeedInput | None = None,
+        entry: EntryInput | None = None,
+        read: bool | None = None,
+        important: TristateFilterInput = None,
+        has_enclosures: bool | None = None,
         feed_tags: TagFilterInput = None,
     ) -> _EFO:
         feed_url = _feed_argument(feed) if feed is not None else None
@@ -396,14 +450,16 @@ class EntryFilterOptions(NamedTuple):
 
         if read not in (None, False, True):
             raise ValueError("read should be one of (None, False, True)")
-        if important not in (None, False, True):
-            raise ValueError("important should be one of (None, False, True)")
+
+        important_filter = tristate_filter_argument(important, 'important')
         if has_enclosures not in (None, False, True):
             raise ValueError("has_enclosures should be one of (None, False, True)")
 
         feed_tag_filter = tag_filter_argument(feed_tags, 'feed_tags')
 
-        return cls(feed_url, entry_id, read, important, has_enclosures, feed_tag_filter)
+        return cls(
+            feed_url, entry_id, read, important_filter, has_enclosures, feed_tag_filter
+        )
 
 
 _FFO = TypeVar('_FFO', bound='FeedFilterOptions')
@@ -413,20 +469,20 @@ class FeedFilterOptions(NamedTuple):
 
     """Options for filtering the results of the "get feed" storage methods."""
 
-    feed_url: Optional[str] = None
+    feed_url: str | None = None
     tags: TagFilter = ()
-    broken: Optional[bool] = None
-    updates_enabled: Optional[bool] = None
-    new: Optional[bool] = None
+    broken: bool | None = None
+    updates_enabled: bool | None = None
+    new: bool | None = None
 
     @classmethod
     def from_args(
-        cls: Type[_FFO],
-        feed: Optional[FeedInput] = None,
+        cls: type[_FFO],
+        feed: FeedInput | None = None,
         tags: TagFilterInput = None,
-        broken: Optional[bool] = None,
-        updates_enabled: Optional[bool] = None,
-        new: Optional[bool] = None,
+        broken: bool | None = None,
+        updates_enabled: bool | None = None,
+        new: bool | None = None,
     ) -> _FFO:
         feed_url = _feed_argument(feed) if feed is not None else None
         tag_filter = tag_filter_argument(tags)
@@ -448,7 +504,7 @@ class NameScheme(_namedtuple_compat):
     separator: str
 
     @classmethod
-    def from_value(cls, value: Mapping[str, str]) -> 'NameScheme':
+    def from_value(cls, value: Mapping[str, str]) -> NameScheme:
         # Use is validation.
         self = cls(**value)
         self.make_reader_name('key')
@@ -458,7 +514,7 @@ class NameScheme(_namedtuple_compat):
     def make_reader_name(self, key: str) -> str:
         return self.reader_prefix + key
 
-    def make_plugin_name(self, plugin_name: str, key: Optional[str] = None) -> str:
+    def make_plugin_name(self, plugin_name: str, key: str | None = None) -> str:
         rv = self.plugin_prefix + plugin_name
         if key is not None:
             rv += self.separator + key
@@ -480,8 +536,8 @@ _NT = TypeVar('_NT', bound=_namedtuple_compat)
 def fix_datetime_tzinfo(
     obj: _NT,
     *names: str,
-    _old: Union[None, timezone, bool] = None,
-    _new: Union[None, timezone] = timezone.utc,
+    _old: None | timezone | bool = None,
+    _new: None | timezone = timezone.utc,
     **kwargs: Any,
 ) -> _NT:
     """For specific optional datetime attributes of an object,
