@@ -1,24 +1,21 @@
+from __future__ import annotations
+
 import functools
 import inspect
 import itertools
 import logging
-import multiprocessing.dummy
 import warnings
+from collections.abc import Iterable
+from collections.abc import Iterator
+from collections.abc import Sequence
 from contextlib import contextmanager
 from functools import wraps
 from queue import Queue
 from typing import Any
 from typing import Callable
 from typing import cast
-from typing import Iterable
-from typing import Iterator
 from typing import no_type_check
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
-from typing import TYPE_CHECKING
 from typing import TypeVar
-from typing import Union
 
 
 FuncType = Callable[..., Any]
@@ -40,8 +37,8 @@ MISSING = MissingType()
 def zero_or_one(
     it: Iterable[_U],
     make_exc: Callable[[], Exception],
-    default: Union[MissingType, _T] = MISSING,
-) -> Union[_U, _T]:
+    default: MissingType | _T = MISSING,
+) -> _U | _T:
     things = list(it)
     if len(things) == 0:
         if isinstance(default, MissingType):
@@ -62,9 +59,9 @@ def exactly_one(it: Iterable[_U]) -> _U:
 
 
 def join_paginated_iter(
-    get_things: Callable[[int, Optional[_T]], Iterable[Tuple[_U, _T]]],
+    get_things: Callable[[int, _T | None], Iterable[tuple[_U, _T]]],
     chunk_size: int,
-    last: Optional[_T] = None,
+    last: _T | None = None,
     limit: int = 0,
 ) -> Iterable[_U]:
     """
@@ -134,8 +131,26 @@ def chunks(n: int, iterable: Iterable[_T]) -> Iterable[Iterable[_T]]:
         yield itertools.chain([first], chunk)
 
 
+def count_consumed(it: Iterable[_T]) -> tuple[Iterable[_T], Callable[[], int]]:
+    consumed = 0
+
+    def wrapper() -> Iterable[_T]:
+        nonlocal consumed
+        for e in it:
+            yield e
+            consumed += 1
+
+    def get_count() -> int:
+        return consumed
+
+    return wrapper(), get_count
+
+
 @contextmanager
 def make_pool_map(workers: int) -> Iterator[F]:
+    # lazy import (https://github.com/lemon24/reader/issues/297)
+    import multiprocessing.dummy
+
     pool = multiprocessing.dummy.Pool(workers)
     try:
         yield wrap_map(pool.imap_unordered, workers)
@@ -199,7 +214,7 @@ class PrefixLogger(logging.LoggerAdapter):  # type: ignore
     def _escape(s: str) -> str:  # pragma: no cover
         return '%%'.join(s.split('%'))
 
-    def process(self, msg: str, kwargs: Any) -> Tuple[str, Any]:  # pragma: no cover
+    def process(self, msg: str, kwargs: Any) -> tuple[str, Any]:  # pragma: no cover
         return ': '.join(tuple(self._escape(p) for p in self.prefixes) + (msg,)), kwargs
 
 
@@ -291,22 +306,7 @@ def _name(thing: object) -> str:
     return '<noname>'
 
 
-# avoid "TypeError: 'type' object is not subscriptable" on Python <= 3.8
-# https://stackoverflow.com/a/48554601
-
-if TYPE_CHECKING:  # pragma: no cover
-    Partial = functools.partial
-else:
-
-    class FakeGenericMeta(type):
-        def __getitem__(self, item):  # noqa: B902
-            return self
-
-    class Partial(functools.partial, metaclass=FakeGenericMeta):
-        pass
-
-
-class BetterStrPartial(Partial[_T]):
+class BetterStrPartial(functools.partial[_T]):
     __slots__ = ()
 
     def __str__(self) -> str:
