@@ -1,26 +1,25 @@
+from __future__ import annotations
+
 import builtins
 import logging
 import numbers
 import warnings
+from collections.abc import Iterable
+from collections.abc import Mapping
+from collections.abc import MutableSequence
 from contextlib import nullcontext
 from datetime import datetime
 from datetime import timezone
 from types import MappingProxyType
 from typing import Any
 from typing import Callable
-from typing import Iterable
 from typing import Literal
-from typing import Mapping
-from typing import MutableSequence
-from typing import Optional
 from typing import overload
-from typing import Tuple
 from typing import TypeVar
-from typing import Union
 
 from ._parser import default_parser
 from ._parser import Parser
-from ._parser import SESSION_TIMEOUT
+from ._requests_utils import DEFAULT_TIMEOUT
 from ._requests_utils import TimeoutType
 from ._search import Search
 from ._storage import Storage
@@ -67,6 +66,7 @@ from .types import MissingType
 from .types import ResourceInput
 from .types import SearchSortOrder
 from .types import TagFilterInput
+from .types import TristateFilterInput
 from .types import UpdatedFeed
 from .types import UpdateResult
 
@@ -87,14 +87,14 @@ FeedsUpdateHook = Callable[['Reader'], None]
 def make_reader(
     url: str,
     *,
-    feed_root: Optional[str] = None,
+    feed_root: str | None = None,
     plugins: Iterable[PluginInput] = DEFAULT_PLUGINS,
-    session_timeout: TimeoutType = SESSION_TIMEOUT,
+    session_timeout: TimeoutType = DEFAULT_TIMEOUT,
     reserved_name_scheme: Mapping[str, str] = DEFAULT_RESERVED_NAME_SCHEME,
-    search_enabled: Union[bool, None, Literal['auto']] = 'auto',
-    _storage: Optional[Storage] = None,
+    search_enabled: bool | None | Literal['auto'] = 'auto',
+    _storage: Storage | None = None,
     _storage_factory: Any = None,
-) -> 'Reader':
+) -> Reader:
     """Create a new :class:`Reader`.
 
     *reader* can optionally parse local files, with the feed URL either
@@ -231,6 +231,10 @@ def make_reader(
         raise ValueError("search_enabled should be one of ('auto', True, False, None)")
 
     parser = default_parser(feed_root, session_timeout=session_timeout)
+    # circular import
+    from . import USER_AGENT
+
+    parser.session_factory.user_agent = USER_AGENT
 
     try:
         name_scheme = NameScheme.from_value(reserved_name_scheme)
@@ -608,14 +612,14 @@ class Reader:
     def get_feeds(
         self,
         *,
-        feed: Optional[FeedInput] = None,
+        feed: FeedInput | None = None,
         tags: TagFilterInput = None,
-        broken: Optional[bool] = None,
-        updates_enabled: Optional[bool] = None,
-        new: Optional[bool] = None,
+        broken: bool | None = None,
+        updates_enabled: bool | None = None,
+        new: bool | None = None,
         sort: FeedSortOrder = 'title',
-        limit: Optional[int] = None,
-        starting_after: Optional[FeedInput] = None,
+        limit: int | None = None,
+        starting_after: FeedInput | None = None,
     ) -> Iterable[Feed]:
         """Get all or some of the feeds.
 
@@ -739,15 +743,15 @@ class Reader:
         feed: FeedInput,
         default: _T,
         /,
-    ) -> Union[Feed, _T]:  # pragma: no cover
+    ) -> Feed | _T:  # pragma: no cover
         ...
 
     def get_feed(
         self,
         feed: FeedInput,
-        default: Union[MissingType, _T] = MISSING,
+        default: MissingType | _T = MISSING,
         /,
-    ) -> Union[Feed, _T]:
+    ) -> Feed | _T:
         """Get a feed.
 
         Like ``next(iter(reader.get_feeds(feed=feed)))``,
@@ -777,11 +781,11 @@ class Reader:
     def get_feed_counts(
         self,
         *,
-        feed: Optional[FeedInput] = None,
+        feed: FeedInput | None = None,
         tags: TagFilterInput = None,
-        broken: Optional[bool] = None,
-        updates_enabled: Optional[bool] = None,
-        new: Optional[bool] = None,
+        broken: bool | None = None,
+        updates_enabled: bool | None = None,
+        new: bool | None = None,
     ) -> FeedCounts:
         """Count all or some of the feeds.
 
@@ -815,7 +819,7 @@ class Reader:
         )
         return self._storage.get_feed_counts(filter_options)
 
-    def set_feed_user_title(self, feed: FeedInput, title: Optional[str], /) -> None:
+    def set_feed_user_title(self, feed: FeedInput, title: str | None, /) -> None:
         """Set a user-defined title for a feed.
 
         Args:
@@ -878,11 +882,11 @@ class Reader:
     def update_feeds(
         self,
         *,
-        feed: Optional[FeedInput] = None,
+        feed: FeedInput | None = None,
         tags: TagFilterInput = None,
-        broken: Optional[bool] = None,
-        updates_enabled: Optional[bool] = True,
-        new: Optional[bool] = None,
+        broken: bool | None = None,
+        updates_enabled: bool | None = True,
+        new: bool | None = None,
         workers: int = 1,
     ) -> None:
         """Update all or some of the feeds.
@@ -960,11 +964,11 @@ class Reader:
     def update_feeds_iter(
         self,
         *,
-        feed: Optional[FeedInput] = None,
+        feed: FeedInput | None = None,
         tags: TagFilterInput = None,
-        broken: Optional[bool] = None,
-        updates_enabled: Optional[bool] = True,
-        new: Optional[bool] = None,
+        broken: bool | None = None,
+        updates_enabled: bool | None = True,
+        new: bool | None = None,
         workers: int = 1,
         _call_feeds_update_hooks: bool = True,
     ) -> Iterable[UpdateResult]:
@@ -1040,7 +1044,7 @@ class Reader:
             for hook in self.after_feeds_update_hooks:
                 hook(self)
 
-    def update_feed(self, feed: FeedInput, /) -> Optional[UpdatedFeed]:
+    def update_feed(self, feed: FeedInput, /) -> UpdatedFeed | None:
         """Update a single feed.
 
         The feed will be updated even if updates are disabled for it.
@@ -1090,15 +1094,15 @@ class Reader:
     def get_entries(
         self,
         *,
-        feed: Optional[FeedInput] = None,
-        entry: Optional[EntryInput] = None,
-        read: Optional[bool] = None,
-        important: Optional[bool] = None,
-        has_enclosures: Optional[bool] = None,
+        feed: FeedInput | None = None,
+        entry: EntryInput | None = None,
+        read: bool | None = None,
+        important: TristateFilterInput = None,
+        has_enclosures: bool | None = None,
         feed_tags: TagFilterInput = None,
         sort: EntrySortOrder = 'recent',
-        limit: Optional[int] = None,
-        starting_after: Optional[EntryInput] = None,
+        limit: int | None = None,
+        starting_after: EntryInput | None = None,
     ) -> Iterable[Entry]:
         """Get all or some of the entries.
 
@@ -1106,20 +1110,27 @@ class Reader:
 
         ``'recent'``
 
-            Most recent first. Currently, that means:
+            Most recent first. That is:
 
-            * by import date for entries published less than 7 days ago
-            * by published date otherwise (if an entry does not have
-              :attr:`~Entry.published`, :attr:`~Entry.updated` is used)
+            * by published date for entries imported on the first update
+              (if an entry does not have :attr:`~Entry.published`,
+              :attr:`~Entry.updated` is used)
+            * by added date for entries imported after that
 
             This is to make sure newly imported entries appear at the top
-            regardless of when the feed says they were published
-            (sometimes, it lies by a day or two).
+            regardless of when the feed says they were published,
+            while not having all the old entries at the top for new feeds.
 
             .. note::
 
                 The algorithm for "recent" is a heuristic and may change over time.
 
+            .. versionchanged:: 3.1
+                Sort entries by added date most of the time,
+                with the exception of those imported on the first update.
+                Previously, entries would be sorted by added
+                only if they were published less than 7 days ago.
+
         ``'random'``
 
             Random order (shuffled). At at most 256 entries will be returned.
@@ -1131,7 +1142,10 @@ class Reader:
             entry (tuple(str, str) or Entry or None):
                 Only return the entry with this (feed URL, entry id) tuple.
             read (bool or None): Only return (un)read entries.
-            important (bool or None): Only return (un)important entries.
+            important (bool or None or str):
+                Only return (un)important entries.
+                For more precise filtering, use one of the
+                :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only return entries that (don't)
                 have enclosures.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
@@ -1161,6 +1175,9 @@ class Reader:
         .. versionadded:: 1.12
             The ``limit`` and ``starting_after`` keyword arguments.
 
+        .. versionchanged:: 3.5
+            The ``important`` argument also accepts string values.
+
         """
 
         # If we ever implement pagination, consider following the guidance in
@@ -1213,15 +1230,15 @@ class Reader:
         entry: EntryInput,
         default: _T,
         /,
-    ) -> Union[Entry, _T]:  # pragma: no cover
+    ) -> Entry | _T:  # pragma: no cover
         ...
 
     def get_entry(
         self,
         entry: EntryInput,
-        default: Union[MissingType, _T] = MISSING,
+        default: MissingType | _T = MISSING,
         /,
-    ) -> Union[Entry, _T]:
+    ) -> Entry | _T:
         """Get an entry.
 
         Like ``next(iter(reader.get_entries(entry=entry)))``,
@@ -1251,11 +1268,11 @@ class Reader:
     def get_entry_counts(
         self,
         *,
-        feed: Optional[FeedInput] = None,
-        entry: Optional[EntryInput] = None,
-        read: Optional[bool] = None,
-        important: Optional[bool] = None,
-        has_enclosures: Optional[bool] = None,
+        feed: FeedInput | None = None,
+        entry: EntryInput | None = None,
+        read: bool | None = None,
+        important: TristateFilterInput = None,
+        has_enclosures: bool | None = None,
         feed_tags: TagFilterInput = None,
     ) -> EntryCounts:
         """Count all or some of the entries.
@@ -1267,7 +1284,10 @@ class Reader:
             entry (tuple(str, str) or Entry or None):
                 Only count the entry with this (feed URL, entry id) tuple.
             read (bool or None): Only count (un)read entries.
-            important (bool or None): Only count (un)important entries.
+            important (bool or None or str):
+                Only count (un)important entries.
+                For more precise filtering, use one of the
+                :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only count entries that (don't)
                 have enclosures.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
@@ -1281,6 +1301,9 @@ class Reader:
 
         .. versionadded:: 1.11
 
+        .. versionchanged:: 3.5
+            The ``important`` argument also accepts string values.
+
         """
 
         filter_options = EntryFilterOptions.from_args(
@@ -1294,14 +1317,14 @@ class Reader:
         entry: EntryInput,
         read: bool,
         /,
-        modified: Union[MissingType, None, datetime] = MISSING,
+        modified: MissingType | None | datetime = MISSING,
     ) -> None:
         """Mark an entry as read or unread,
         possibly with a custom timestamp.
 
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
-            read (bool): Mark the entry as read if true (default),
+            read (bool): Mark the entry as read if true,
                 and as unread otherwise.
             modified (datetime or None):
                 Set :attr:`~Entry.read_modified` to this.
@@ -1318,8 +1341,15 @@ class Reader:
         .. versionchanged:: 3.0
             The ``entry`` and ``read`` arguments are now positional-only.
 
+        .. versionchanged:: 3.5
+            Do not coerce ``read`` to :class:`bool` anymore,
+            require it to be :const:`True` or :const:`False`.
+
         """
-        modified_naive: Optional[datetime]
+        if read not in (True, False):
+            raise ValueError("read should be one of (True, False)")
+
+        modified_naive: datetime | None
         if isinstance(modified, MissingType):
             modified_naive = self._now()
         elif modified is None:
@@ -1328,7 +1358,7 @@ class Reader:
             modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
 
         feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_read(feed_url, entry_id, bool(read), modified_naive)
+        self._storage.mark_as_read(feed_url, entry_id, read, modified_naive)
 
     def mark_entry_as_read(self, entry: EntryInput, /) -> None:
         """Mark an entry as read.
@@ -1375,17 +1405,17 @@ class Reader:
     def set_entry_important(
         self,
         entry: EntryInput,
-        important: bool,
+        important: bool | None,
         /,
-        modified: Union[MissingType, None, datetime] = MISSING,
+        modified: MissingType | None | datetime = MISSING,
     ) -> None:
         """Mark an entry as important or unimportant,
         possibly with a custom timestamp.
 
         Args:
             entry (tuple(str, str) or Entry): (feed URL, entry id) tuple.
-            important (bool): Mark the entry as important if true (default),
-                and as unimportant otherwise.
+            important (bool or None): Mark the entry as important if true,
+                as unimportant if false, or as not set if none.
             modified (datetime or None):
                 Set :attr:`~Entry.important_modified` to this.
                 Naive datetimes are normalized by passing them to
@@ -1401,8 +1431,18 @@ class Reader:
         .. versionchanged:: 3.0
             The ``entry`` and ``important`` arguments are now positional-only.
 
+        .. versionchanged:: 3.5
+            ``important`` can now be :const:`None`.
+
+        .. versionchanged:: 3.5
+            Do not coerce ``important`` to :class:`bool` anymore,
+            require it to be :const:`True` or :const:`False` or :const:`None`.
+
         """
-        modified_naive: Optional[datetime]
+        if important not in (True, False, None):
+            raise ValueError("important should be one of (True, False, None)")
+
+        modified_naive: datetime | None
         if isinstance(modified, MissingType):
             modified_naive = self._now()
         elif modified is None:
@@ -1411,9 +1451,7 @@ class Reader:
             modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
 
         feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_important(
-            feed_url, entry_id, bool(important), modified_naive
-        )
+        self._storage.mark_as_important(feed_url, entry_id, important, modified_naive)
 
     def mark_entry_as_important(self, entry: EntryInput, /) -> None:
         """Mark an entry as important.
@@ -1457,22 +1495,6 @@ class Reader:
         """
         return self.set_entry_important(entry, False)
 
-    def _mark_entry_as_dont_care(self, entry: EntryInput, /) -> None:
-        """Mark an entry as read and unimportant at the same time,
-        resulting in the same read_modified and important_modified.
-
-        This method becoming public is pending on #254.
-
-        Presumably, we could just use mark_entry_as_{read,important} instead
-        and get the slightly different timestamps,
-        but it's likely better to collect more accurate data.
-
-        """
-        modified_naive = self._now()
-        feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_read(feed_url, entry_id, True, modified_naive)
-        self._storage.mark_as_important(feed_url, entry_id, False, modified_naive)
-
     def add_entry(self, entry: Any, /) -> None:
         """Add a new entry to an existing feed.
 
@@ -1533,12 +1555,13 @@ class Reader:
         # https://gist.github.com/lemon24/047f71abe76c47661634459eada7b50a#file-01-typing-py
 
         now = self._now()
-
+        entry_data = entry_data_from_obj(entry)
         intent = EntryUpdateIntent(
-            entry=entry_data_from_obj(entry),
+            entry=entry_data,
             last_updated=now,
             first_updated=now,
             first_updated_epoch=now,
+            recent_sort=entry_data.published or entry_data.updated or now,
             added_by='user',
         )
 
@@ -1640,15 +1663,15 @@ class Reader:
         query: str,
         /,
         *,
-        feed: Optional[FeedInput] = None,
-        entry: Optional[EntryInput] = None,
-        read: Optional[bool] = None,
-        important: Optional[bool] = None,
-        has_enclosures: Optional[bool] = None,
+        feed: FeedInput | None = None,
+        entry: EntryInput | None = None,
+        read: bool | None = None,
+        important: TristateFilterInput = None,
+        has_enclosures: bool | None = None,
         feed_tags: TagFilterInput = None,
         sort: SearchSortOrder = 'relevant',
-        limit: Optional[int] = None,
-        starting_after: Optional[EntryInput] = None,
+        limit: int | None = None,
+        starting_after: EntryInput | None = None,
     ) -> Iterable[EntrySearchResult]:
         """Get entries matching a full-text search query.
 
@@ -1665,6 +1688,12 @@ class Reader:
 
             .. versionadded:: 1.4
 
+            .. versionchanged:: 3.1
+                Sort entries by added date most of the time,
+                with the exception of those imported on the first update.
+                Previously, entries would be sorted by added
+                only if they were published less than 7 days ago.
+
         ``'random'``
 
             Random order (shuffled). At at most 256 entries will be returned.
@@ -1704,7 +1733,10 @@ class Reader:
             entry (tuple(str, str) or Entry or None):
                 Only search for the entry with this (feed URL, entry id) tuple.
             read (bool or None): Only search (un)read entries.
-            important (bool or None): Only search (un)important entries.
+            important (bool or None or str):
+                Only search (un)important entries.
+                For more precise filtering, use one of the
+                :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only search entries that (don't)
                 have enclosures.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
@@ -1740,6 +1772,9 @@ class Reader:
         .. versionchanged:: 3.0
             The ``query`` argument is now positional-only.
 
+        .. versionchanged:: 3.5
+            The ``important`` argument also accepts string values.
+
         """
         filter_options = EntryFilterOptions.from_args(
             feed, entry, read, important, has_enclosures, feed_tags
@@ -1770,11 +1805,11 @@ class Reader:
         query: str,
         /,
         *,
-        feed: Optional[FeedInput] = None,
-        entry: Optional[EntryInput] = None,
-        read: Optional[bool] = None,
-        important: Optional[bool] = None,
-        has_enclosures: Optional[bool] = None,
+        feed: FeedInput | None = None,
+        entry: EntryInput | None = None,
+        read: bool | None = None,
+        important: TristateFilterInput = None,
+        has_enclosures: bool | None = None,
         feed_tags: TagFilterInput = None,
     ) -> EntrySearchCounts:
         """Count entries matching a full-text search query.
@@ -1789,7 +1824,10 @@ class Reader:
             feed (str or tuple(str) or Feed or None): Only count the entries for this feed.
             entry (tuple(str, str) or Entry or None):
                 Only count the entry with this (feed URL, entry id) tuple.
-            read (bool or None): Only count (un)read entries.
+            read (bool or None or str):
+                Only count (un)read entries.
+                For more precise filtering, use one of the
+                :data:`~reader.types.TristateFilterInput` string filters.
             important (bool or None): Only count (un)important entries.
             has_enclosures (bool or None): Only count entries that (don't)
                 have enclosures.
@@ -1810,6 +1848,9 @@ class Reader:
         .. versionchanged:: 3.0
             The ``query`` argument is now positional-only.
 
+        .. versionchanged:: 3.5
+            The ``important`` argument also accepts string values.
+
         """
 
         filter_options = EntryFilterOptions.from_args(
@@ -1819,8 +1860,8 @@ class Reader:
         return self._search.search_entry_counts(query, now, filter_options)
 
     def get_tags(
-        self, resource: ResourceInput, /, *, key: Optional[str] = None
-    ) -> Iterable[Tuple[str, JSONType]]:
+        self, resource: ResourceInput, /, *, key: str | None = None
+    ) -> Iterable[tuple[str, JSONType]]:
         """Get all or some tags of a resource as ``(key, value)`` pairs.
 
         `resource` can have one of the following types:
@@ -1930,16 +1971,16 @@ class Reader:
         key: str,
         default: _T,
         /,
-    ) -> Union[JSONType, _T]:  # pragma: no cover
+    ) -> JSONType | _T:  # pragma: no cover
         ...
 
     def get_tag(
         self,
         resource: ResourceInput,
         key: str,
-        default: Union[MissingType, _T] = MISSING,
+        default: MissingType | _T = MISSING,
         /,
-    ) -> Union[JSONType, _T]:
+    ) -> JSONType | _T:
         """Get the value of this resource tag.
 
         Like ``next(iter(reader.get_tags(resource, key=key)))[1]``,
@@ -1994,7 +2035,7 @@ class Reader:
         self,
         resource: ResourceInput,
         key: str,
-        value: Union[JSONType, MissingType] = MISSING,
+        value: JSONType | MissingType = MISSING,
         /,
     ) -> None:
         """Set the value of this resource tag.
@@ -2093,7 +2134,7 @@ class Reader:
     def make_plugin_reserved_name(
         self,
         plugin_name: str,
-        key: Optional[str] = None,
+        key: str | None = None,
         /,
     ) -> str:
         """Create a plugin-reserved tag name.
