@@ -25,6 +25,13 @@ tag to something like::
     Feeds using the old metadata, ``.reader.mark_as_read``,
     will be migrated automatically on update until `reader` 3.0.
 
+.. versionchanged:: 3.5
+    Don't set :attr:`~reader.Entry.read_modified` and
+    :attr:`~reader.Entry.important_modified` anymore;
+    because :attr:`~reader.Entry.important` is now optional,
+    ``important = False`` is enough to mark an entry as unimportant.
+    Old unimportant entries will be migrated automatically.
+
 
 .. todo::
 
@@ -41,6 +48,7 @@ tag to something like::
 import logging
 import re
 
+from reader.exceptions import EntryNotFoundError
 from reader.types import EntryUpdateStatus
 
 # avoid circular imports
@@ -74,10 +82,16 @@ def _mark_as_read(reader, entry, status):
     key = reader.make_reader_reserved_name(_CONFIG_TAG)
     patterns = _get_config(reader, entry.feed_url, key, 'title')
 
-    for pattern in patterns or ():
-        if re.search(pattern, entry.title):
-            reader._mark_entry_as_dont_care(entry)
-            return
+    try:
+        for pattern in patterns or ():
+            if re.search(pattern, entry.title or ''):
+                reader.set_entry_read(entry, True, None)
+                reader.set_entry_important(entry, False, None)
+                return
+    except EntryNotFoundError as e:
+        if entry.resource_id != e.resource_id:  # pragma: no cover
+            raise
+        log.info("entry %r was deleted, skipping", entry.resource_id)
 
 
 def init_reader(reader):
