@@ -1,26 +1,24 @@
+from __future__ import annotations
+
 import dataclasses
 import enum
 import re
 import traceback
+from collections.abc import Iterable
+from collections.abc import Mapping
+from collections.abc import Sequence
 from dataclasses import dataclass
+from dataclasses import field
 from datetime import datetime
 from types import MappingProxyType
 from typing import Any
 from typing import Callable
 from typing import cast
-from typing import Dict
-from typing import Iterable
-from typing import List
 from typing import Literal
-from typing import Mapping
 from typing import NamedTuple
-from typing import Optional
 from typing import overload
 from typing import Protocol
 from typing import runtime_checkable
-from typing import Sequence
-from typing import Tuple
-from typing import Type
 from typing import TypeVar
 from typing import Union
 
@@ -42,18 +40,19 @@ class _namedtuple_compat:
     # TODO: can we get rid of _namedtuple_compat?
 
     @classmethod
-    def _make(cls: Type[_T], iterable: Iterable[Any]) -> _T:
+    def _make(cls: type[_T], iterable: Iterable[Any]) -> _T:
         iterable = tuple(iterable)
-        attrs_len = len(dataclasses.fields(cls))
+        attrs_len = len(dataclasses.fields(cls))  # type: ignore[arg-type]
         if len(iterable) != attrs_len:
             raise TypeError(
                 'Expected %d arguments, got %d' % (attrs_len, len(iterable))
             )
         return cls(*iterable)
 
-    _replace = dataclasses.replace
+    def _replace(self: _T, **kargs: Any) -> _T:
+        return dataclasses.replace(self, **kargs)  # type: ignore[type-var]
 
-    def _asdict(self) -> Dict[str, Any]:
+    def _asdict(self) -> dict[str, Any]:
         return dict(self.__dict__)
 
 
@@ -86,21 +85,21 @@ class Feed(_namedtuple_compat):
     url: str
 
     #: The date the feed was last updated, according to the feed.
-    updated: Optional[datetime] = None
+    updated: datetime | None = None
 
     #: The title of the feed.
-    title: Optional[str] = None
+    title: str | None = None
 
     #: The URL of a page associated with the feed.
-    link: Optional[str] = None
+    link: str | None = None
 
     #: The author of the feed.
-    author: Optional[str] = None
+    author: str | None = None
 
     #: A description or subtitle for the feed.
     #:
     #: .. versionadded:: 2.4
-    subtitle: Optional[str] = None
+    subtitle: str | None = None
 
     #: The feed type and version.
     #:
@@ -121,10 +120,10 @@ class Feed(_namedtuple_compat):
     #: Plugins may add other versions.
     #:
     #: .. versionadded:: 2.4
-    version: Optional[str] = None
+    version: str | None = None
 
     #: User-defined feed title.
-    user_title: Optional[str] = None
+    user_title: str | None = None
 
     # added is required, but we want it after feed data; the cast is for mypy.
 
@@ -136,12 +135,12 @@ class Feed(_namedtuple_compat):
     #: The date when the feed was last retrieved by reader.
     #:
     #: .. versionadded:: 1.3
-    last_updated: Optional[datetime] = None
+    last_updated: datetime | None = None
 
     #: If a :exc:`ParseError` happend during the last update, its cause.
     #:
     #: .. versionadded:: 1.3
-    last_exception: Optional['ExceptionInfo'] = None
+    last_exception: ExceptionInfo | None = None
 
     #: Whether updates are enabled for this feed.
     #:
@@ -149,7 +148,7 @@ class Feed(_namedtuple_compat):
     updates_enabled: bool = True
 
     @property
-    def resource_id(self) -> Tuple[str]:
+    def resource_id(self) -> tuple[str]:
         """Alias for (:attr:`~url`,).
 
         .. versionadded:: 2.17
@@ -183,7 +182,7 @@ class ExceptionInfo(_namedtuple_compat):
     traceback_str: str
 
     @classmethod
-    def from_exception(cls: Type[_EI], exc: BaseException) -> _EI:
+    def from_exception(cls: type[_EI], exc: BaseException) -> _EI:
         return cls(
             f'{type(exc).__module__}.{type(exc).__qualname__}',
             str(exc),
@@ -231,30 +230,30 @@ class Entry(_namedtuple_compat):
     #:  Is now :const:`None` if missing in the feed;
     #:  use :attr:`updated_not_none` for the pre-2.5 behavior.
     #:
-    updated: Optional[datetime] = None
+    updated: datetime | None = None
 
     #: The title of the entry.
-    title: Optional[str] = None
+    title: str | None = None
 
     #: The URL of a page associated with the entry.
-    link: Optional[str] = None
+    link: str | None = None
 
     #: The author of the feed.
-    author: Optional[str] = None
+    author: str | None = None
 
     #: The date the entry was published, according to the feed.
-    published: Optional[datetime] = None
+    published: datetime | None = None
 
     #: A summary of the entry.
-    summary: Optional[str] = None
+    summary: str | None = None
 
     #: Full content of the entry.
     #: A sequence of :class:`Content` objects.
-    content: Sequence['Content'] = ()
+    content: Sequence[Content] = ()
 
     #: External files associated with the entry.
     #: A sequence of :class:`Enclosure` objects.
-    enclosures: Sequence['Enclosure'] = ()
+    enclosures: Sequence[Enclosure] = ()
 
     #: Whether the entry was read or not.
     read: bool = False
@@ -264,17 +263,23 @@ class Entry(_namedtuple_compat):
     #: or the entry predates the date being recorded.
     #:
     #: .. versionadded:: 2.2
-    read_modified: Optional[datetime] = None
+    read_modified: datetime | None = None
 
     #: Whether the entry is important or not.
-    important: bool = False
+    #: :const:`None` means not set.
+    #: :const:`False` means "explicitly unimportant".
+    #:
+    #: .. versionchanged:: 3.5
+    #:  :attr:`important` is now an optional :class:`bool`,
+    #:  and defaults to :const:`None`.
+    important: bool | None = None
 
     #: The date when :attr:`important` was last set by the user;
     #: :const:`None` if that never happened,
     #: or the entry predates the date being recorded.
     #:
     #: .. versionadded:: 2.2
-    important_modified: Optional[datetime] = None
+    important_modified: datetime | None = None
 
     #: The date when the entry was added (first updated) to reader.
     #:
@@ -313,7 +318,7 @@ class Entry(_namedtuple_compat):
     feed: Feed = cast(Feed, None)
 
     @property
-    def resource_id(self) -> Tuple[str, str]:
+    def resource_id(self) -> tuple[str, str]:
         """Alias for (:attr:`~feed_url`, :attr:`~id`).
 
         .. versionadded:: 2.17
@@ -334,7 +339,7 @@ class Entry(_namedtuple_compat):
         """
         return self.updated or self.added
 
-    def get_content(self, *, prefer_summary: bool = False) -> Optional['Content']:
+    def get_content(self, *, prefer_summary: bool = False) -> Content | None:
         """Return a text content OR the summary.
 
         Prefer HTML content, when available.
@@ -362,10 +367,10 @@ class Content(_namedtuple_compat):
     value: str
 
     #: The content type.
-    type: Optional[str] = None
+    type: str | None = None
 
     #: The content language.
-    language: Optional[str] = None
+    language: str | None = None
 
     @property
     def is_html(self) -> bool:
@@ -385,7 +390,7 @@ _HTML_CONTENT_TYPES = {'text/html', 'text/xhtml'}
 _HTML_CONTENT_TYPE = 'text/html'
 
 
-def _get_entry_content(entry: Entry, prefer_summary: bool = False) -> Optional[Content]:
+def _get_entry_content(entry: Entry, prefer_summary: bool = False) -> Content | None:
     # TODO: Make this public; Entry should be a protocol, Content should be generic.
     # TODO: Use the type from .summary_detail (when we get it).
 
@@ -408,16 +413,16 @@ class Enclosure(_namedtuple_compat):
 
     """Data type representing an external file."""
 
-    # WARNING: When changing attributes, keep content_from_obj in sync.
+    # WARNING: When changing attributes, keep enclosure_from_obj in sync.
 
     #: The file URL.
     href: str
 
     #: The file content type.
-    type: Optional[str] = None
+    type: str | None = None
 
     #: The file length.
-    length: Optional[int] = None
+    length: int | None = None
 
 
 _HS = TypeVar('_HS', bound='HighlightedString')
@@ -476,7 +481,7 @@ class HighlightedString:
         return self.value
 
     @classmethod
-    def extract(cls: Type[_HS], text: str, before: str, after: str) -> _HS:
+    def extract(cls: type[_HS], text: str, before: str, after: str) -> _HS:
         """Extract highlights with before/after markers from text.
 
         >>> HighlightedString.extract( '>one< two', '>', '<')
@@ -546,7 +551,7 @@ class HighlightedString:
         self,
         before: str,
         after: str,
-        func: Optional[Callable[[str], str]] = None,
+        func: Callable[[str], str] | None = None,
     ) -> str:
         """Apply before/end markers on the highlighted string.
 
@@ -607,16 +612,20 @@ class EntrySearchResult(_namedtuple_compat):
 
     #: Matching entry metadata, in arbitrary order.
     #: Currently entry.title and entry.feed.user_title/.title.
-    metadata: Mapping[str, HighlightedString] = MappingProxyType({})
+    metadata: Mapping[str, HighlightedString] = field(
+        default_factory=lambda: MappingProxyType({}),
+    )
 
     #: Matching entry content, sorted by relevance.
     #: Any of entry.summary and entry.content[].value.
-    content: Mapping[str, HighlightedString] = MappingProxyType({})
+    content: Mapping[str, HighlightedString] = field(
+        default_factory=lambda: MappingProxyType({}),
+    )
 
     # TODO: entry: Optional[Entry]; model it through typing if possible
 
     @property
-    def resource_id(self) -> Tuple[str, str]:
+    def resource_id(self) -> tuple[str, str]:
         """Alias for (:attr:`~feed_url`, :attr:`~id`).
 
         .. versionadded:: 2.17
@@ -665,8 +674,8 @@ SearchSortOrder = Literal['relevant', 'recent', 'random']
 
 # https://github.com/python/typing/issues/182
 # TODO: allow JSONType to be str, int, ...
-JSONValue = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]
-JSONType = Union[Dict[str, JSONValue], List[JSONValue]]
+JSONValue = Union[str, int, float, bool, None, dict[str, Any], list[Any]]
+JSONType = Union[dict[str, JSONValue], list[JSONValue]]
 
 
 # Using protocols here so we have both duck typing and type checking.
@@ -696,13 +705,13 @@ class EntryLike(Protocol):
 
 
 # https://github.com/lemon24/reader/issues/266#issuecomment-1013739526
-GlobalInput = Tuple[()]
+GlobalInput = tuple[()]
 FeedInput = Union[str, FeedLike]
-EntryInput = Union[Tuple[str, str], EntryLike]
+EntryInput = Union[tuple[str, str], EntryLike]
 ResourceInput = Union[GlobalInput, FeedInput, EntryInput]
-AnyResourceInput = Union[ResourceInput, None, Tuple[None], Tuple[None, None]]
-ResourceId = Union[Tuple[()], Tuple[str], Tuple[str, str]]
-AnyResourceId = Union[ResourceId, None, Tuple[None], Tuple[None, None]]
+AnyResourceInput = Union[ResourceInput, None, tuple[None], tuple[None, None]]
+ResourceId = Union[tuple[()], tuple[str], tuple[str, str]]
+AnyResourceId = Union[ResourceId, None, tuple[None], tuple[None, None]]
 
 
 def _feed_argument(feed: FeedInput) -> str:
@@ -717,7 +726,7 @@ def _feed_argument(feed: FeedInput) -> str:
     raise ValueError(f'invalid feed argument: {feed!r}')
 
 
-def _entry_argument(entry: EntryInput) -> Tuple[str, str]:
+def _entry_argument(entry: EntryInput) -> tuple[str, str]:
     if isinstance(entry, EntryLike):
         rv = _feed_argument(entry.feed_url), entry.id
     elif isinstance(entry, tuple) and len(entry) == 2:
@@ -732,17 +741,17 @@ def _entry_argument(entry: EntryInput) -> Tuple[str, str]:
 
 
 @overload
-def _resource_argument(resource: GlobalInput) -> Tuple[()]:
+def _resource_argument(resource: GlobalInput) -> tuple[()]:
     ...  # pragma: no cover
 
 
 @overload
-def _resource_argument(resource: FeedInput) -> Tuple[str]:
+def _resource_argument(resource: FeedInput) -> tuple[str]:
     ...  # pragma: no cover
 
 
 @overload
-def _resource_argument(resource: EntryInput) -> Tuple[str, str]:
+def _resource_argument(resource: EntryInput) -> tuple[str, str]:
     ...  # pragma: no cover
 
 
@@ -766,6 +775,42 @@ TagFilterInput = Union[
     None, bool, Sequence[Union[str, bool, Sequence[Union[str, bool]]]]
 ]
 
+#: Possible values for options that filter items by an optional boolean
+#: attribute (one that can be either true, false, or not set).
+#:
+#: :const:`None` selects all items.
+#: :const:`True` and :const:`False` select items based of the attribute's
+#: truth value (a :const:`None` attribute is treated as false).
+#:
+#: For more precise filtering, use one of the following string filters:
+#:
+#: ==================== =============== =======================
+#: attribute values     string filter   optional bool filter
+#: ==================== =============== =======================
+#: True                 istrue          True
+#: False                isfalse
+#: None                 notset
+#: False, None          nottrue         False
+#: True, None           notfalse
+#: True, False          isset
+#: True, False, None    any             None
+#: ==================== =============== =======================
+#:
+#: .. versionadded:: 3.5
+#:
+TristateFilterInput = Literal[
+    None,
+    True,
+    False,
+    'istrue',
+    'isfalse',
+    'notset',
+    'nottrue',
+    'notfalse',
+    'isset',
+    'any',
+]
+
 
 @dataclass(frozen=True)
 class FeedCounts(_namedtuple_compat):
@@ -777,13 +822,13 @@ class FeedCounts(_namedtuple_compat):
     """
 
     #: Total number of feeds.
-    total: Optional[int] = None
+    total: int | None = None
 
     #: Number of broken feeds.
-    broken: Optional[int] = None
+    broken: int | None = None
 
     #: Number of feeds that have updates enabled.
-    updates_enabled: Optional[int] = None
+    updates_enabled: int | None = None
 
 
 @dataclass(frozen=True)
@@ -796,16 +841,16 @@ class EntryCounts(_namedtuple_compat):
     """
 
     #: Total number of entries.
-    total: Optional[int] = None
+    total: int | None = None
 
     #: Number of read entries.
-    read: Optional[int] = None
+    read: int | None = None
 
     #: Number of important entries.
-    important: Optional[int] = None
+    important: int | None = None
 
     #: Number of entries that have enclosures.
-    has_enclosures: Optional[int] = None
+    has_enclosures: int | None = None
 
     # TODO: make averages a rich tuple
     # https://github.com/lemon24/reader/issues/249#issuecomment-894624989
@@ -814,7 +859,7 @@ class EntryCounts(_namedtuple_compat):
     #:
     #: .. versionadded:: 2.1
     #:
-    averages: Optional[Tuple[float, float, float]] = None
+    averages: tuple[float, float, float] | None = None
 
 
 @dataclass(frozen=True)
@@ -834,22 +879,22 @@ class EntrySearchCounts(_namedtuple_compat):
     # related to search stuff (what matched etc.)
 
     #: Total number of entries.
-    total: Optional[int] = None
+    total: int | None = None
 
     #: Number of read entries.
-    read: Optional[int] = None
+    read: int | None = None
 
     #: Number of important entries.
-    important: Optional[int] = None
+    important: int | None = None
 
     #: Number of entries that have enclosures.
-    has_enclosures: Optional[int] = None
+    has_enclosures: int | None = None
 
     #: Average entries per day during the last 1, 3, 12 months, as a 3-tuple.
     #:
     #: .. versionadded:: 2.1
     #:
-    averages: Optional[Tuple[float, float, float]] = None
+    averages: tuple[float, float, float] | None = None
 
 
 @dataclass(frozen=True)
@@ -868,12 +913,34 @@ class UpdatedFeed:
 
     #: The number of new entries
     #: (entries that did not previously exist in storage).
-    new: int
+    #:
+    #: .. versionchanged:: 3.2
+    #:  This field is now optional, and defaults to 0.
+    new: int = 0
 
     #: The number of modified entries
     #: (entries that existed in storage,
     #: but had different data than the corresponding feed file entry.)
-    modified: int
+    #:
+    #: .. versionchanged:: 3.2
+    #:  This field is now optional, and defaults to 0.
+    modified: int = 0
+
+    #: The number of unmodified entries
+    #: (entries that existed in storage,
+    #: but had the same data in the corresponding feed file entry.)
+    #:
+    #: .. versionadded:: 3.2
+    unmodified: int = 0
+
+    @property
+    def total(self) -> int:
+        """The total number of entries in the retrieved feed.
+
+        .. versionadded:: 3.2
+
+        """
+        return self.new + self.modified + self.unmodified
 
 
 class UpdateResult(NamedTuple):
@@ -901,7 +968,7 @@ class UpdateResult(NamedTuple):
     #:
     #:  If there was an error while updating the feed.
     #:
-    value: Union[UpdatedFeed, None, ReaderError]
+    value: UpdatedFeed | None | ReaderError
 
     # The exception type is ReaderError and not ParseError
     # to allow suppressing new errors without breaking the API:
@@ -912,7 +979,7 @@ class UpdateResult(NamedTuple):
     # but that's not necessarily by design.
 
     @property
-    def updated_feed(self) -> Optional[UpdatedFeed]:
+    def updated_feed(self) -> UpdatedFeed | None:
         """The updated feed, if the update was successful, :const:`None` otherwise.
 
         .. versionadded:: 2.1
@@ -921,7 +988,7 @@ class UpdateResult(NamedTuple):
         return self.value if not isinstance(self.value, Exception) else None
 
     @property
-    def error(self) -> Optional[ReaderError]:
+    def error(self) -> ReaderError | None:
         """The exception, if there was an error, :const:`None` otherwise.
 
         .. versionadded:: 2.1
