@@ -7,8 +7,8 @@ from reader._types import FeedData
 
 
 feed = FeedData(
-    url='{}relative.rss'.format(url_base),
-    link='{}file.html'.format(rel_base),
+    url=f'{url_base}relative.rss',
+    link=f'{rel_base}file.html',
     version='rss20',
 )
 
@@ -17,8 +17,8 @@ entries = [
         feed_url=feed.url,
         id='7bd204c6-1655-4c27-aeee-53f933c5395f',
         updated=None,
-        link='{}blog/post/1'.format(rel_base),
-        summary='one <a href="{}target">two</a> three'.format(rel_base),
+        link=f'{rel_base}blog/post/1',
+        summary=f'one <a href="{rel_base}target">two</a> three',
         content=(
             Content(
                 value='<script>evil</script> content', type='text/plain', language=None
