@@ -25,10 +25,14 @@ def _make_feed(number, updated=None, **kwargs):
 
 
 def _make_entry(feed_number, number, updated=None, **kwargs):
+    if isinstance(number, str):
+        entry_number = number
+    else:
+        # evals to tuple
+        entry_number = f'{feed_number}, {number}'
     return EntryData(
         f'{feed_number}',
-        # evals to tuple
-        f'{feed_number}, {number}',
+        entry_number,
         updated,
         kwargs.pop('title', f'Entry #{number}'),
         kwargs.pop('link', f'http://www.example.com/entries/{number}'),
@@ -64,19 +68,21 @@ class Parser:
         raise NotImplementedError
 
     parallel = reader._parser.Parser.parallel
-    persistent_session = staticmethod(nullcontext)
+
+    class session_factory:
+        persistent = staticmethod(nullcontext)
 
     def retrieve(self, url, http_etag, http_last_modified, is_parallel):
         return nullcontext(RetrieveResult(BytesIO(b'opaque')))
 
     def parse(self, url, result):
-        assert result.file.read() == b'opaque', result
+        assert result.resource.read() == b'opaque', result
 
         for feed_number, feed in self.feeds.items():
             if feed.url == url:
                 break
         else:
-            raise RuntimeError("unkown feed: {}".format(url))
+            raise RuntimeError(f"unkown feed: {url}")
 
         feed = fix_datetime_tzinfo(feed, 'updated', _old=self.tzinfo, _new=None)
 
