@@ -164,8 +164,8 @@ def test_plugin(make_reader):
 
     reader.update_feeds()
 
-    assert set((e.id, e.read, e.important) for e in reader.get_entries()) == {
-        t + (False,)
+    assert {(e.id, e.read, e.important) for e in reader.get_entries()} == {
+        t + (None,)
         for t in {
             # remain untouched
             (old.id, False),
@@ -263,18 +263,18 @@ def test_plugin_once(make_reader, db_path, monkeypatch, tags):
     reader.update_feeds()
 
     # nothing changes without tag
-    assert set((e.id, e.read, e.important) for e in reader.get_entries()) == {
-        (different_title.id, False, False),
-        (no_content_match.id, False, False),
-        (read_one.id, True, False),
-        (read_two.id, False, False),
-        (read_three.id, False, False),
-        (important_one.id, False, False),
+    assert {(e.id, e.read, e.important) for e in reader.get_entries()} == {
+        (different_title.id, False, None),
+        (no_content_match.id, False, None),
+        (read_one.id, True, None),
+        (read_two.id, False, None),
+        (read_three.id, False, None),
+        (important_one.id, False, None),
         (important_two.id, False, True),
-        (important_three.id, False, False),
-        (unread_one.id, False, False),
-        (unread_two.id, False, False),
-        (unread_three.id, False, False),
+        (important_three.id, False, None),
+        (unread_one.id, False, None),
+        (unread_two.id, False, None),
+        (unread_three.id, False, None),
     }
 
     for tag in tags:
@@ -288,23 +288,23 @@ def test_plugin_once(make_reader, db_path, monkeypatch, tags):
     # 'once' (the strictest) has priority
     if '.reader.dedupe.once' in tags:
         expected = {
-            (different_title.id, False, False),
-            (no_content_match.id, False, False),
+            (different_title.id, False, None),
+            (no_content_match.id, False, None),
             # delete read_one, read_two (older last_updated)
-            (read_three.id, True, False),
+            (read_three.id, True, None),
             # delete important_one, important_two have (older last_updated)
             (important_three.id, False, True),
             # delete unread_one, unread_two (older last_updated)
-            (unread_three.id, False, False),
+            (unread_three.id, False, None),
         }
     else:
         expected = {
-            (different_title.id, False, False),
+            (different_title.id, False, None),
             # delete all others (older last_updated)
             (unread_three.id, True, True),
         }
 
-    assert set((e.id, e.read, e.important) for e in reader.get_entries()) == expected
+    assert {(e.id, e.read, e.important) for e in reader.get_entries()} == expected
 
 
 READ_MODIFIED_COPYING_DATA = [
@@ -489,6 +489,60 @@ IMPORTANT_MODIFIED_COPYING_DATA = [
             (9, True, None),
         ],
     ),
+    # None vs False combos (subset of the above)
+    # TODO: find a better way?
+    # none set, modified (last has modified)
+    (
+        [
+            (1, None, datetime(2010, 1, 1)),
+            (9, None, datetime(2010, 1, 9)),
+        ],
+        [
+            (9, None, datetime(2010, 1, 1)),
+        ],
+    ),
+    # old unimportant, modified; new unset
+    (
+        [
+            (1, False, datetime(2010, 1, 1)),
+            (9, None, None),
+        ],
+        [
+            (9, False, datetime(2010, 1, 1)),
+        ],
+    ),
+    # new unimportant, old modified
+    (
+        [
+            (1, None, datetime(2010, 1, 1)),
+            (9, False, None),
+        ],
+        [
+            (9, False, None),
+        ],
+    ),
+    # None vs True combos (subset of the above)
+    # TODO: find a better way?
+    # old important, modified; new unset
+    (
+        [
+            (1, True, datetime(2010, 1, 1)),
+            (9, None, None),
+        ],
+        [
+            (9, True, datetime(2010, 1, 1)),
+        ],
+    ),
+    # new important, old modified
+    (
+        [
+            (1, None, datetime(2010, 1, 1)),
+            (9, True, None),
+        ],
+        [
+            (9, True, None),
+        ],
+    ),
 ]
 
 
@@ -669,3 +723,57 @@ def test_entry_tags_copying(make_reader, db_path, tags):
     assert not list(reader.get_tag_keys(one))
     assert not list(reader.get_tag_keys(two))
     assert dict(reader.get_tags(three)) == expected_tags
+
+
+# TODO: with_maybe_published_or_updated
+def test_recent_sort_copying(make_reader, db_path):
+    reader = make_reader(db_path)
+    reader._parser = parser = Parser()
+    reader.add_feed(parser.feed(1))
+
+    parser.entry(1, 1, title='title', summary='summary')
+    reader._now = lambda: naive_datetime(2010, 1, 10)
+    reader.update_feeds()
+
+    parser.entry(1, 2, title='title', summary='summary')
+    parser.entry(1, 3, title='other')
+    reader._now = lambda: naive_datetime(2010, 1, 20)
+    reader.update_feeds()
+
+    reader = make_reader(db_path, plugins=['reader.entry_dedupe'])
+    reader._parser = parser
+
+    del parser.entries[1][1]
+    del parser.entries[1][2]
+    four = parser.entry(1, 4, title='title', summary='summary')
+    reader._now = lambda: naive_datetime(2010, 2, 1)
+    reader.update_feeds()
+
+    assert [eval(e.id)[1] for e in reader.get_entries(sort='recent')] == [3, 4]
+
+    actual_recent_sort = reader._storage.get_entry_recent_sort(four.resource_id)
+    assert actual_recent_sort == naive_datetime(2010, 1, 10)
+
+
+@pytest.mark.parametrize('update_after_one', [False, True])
+def test_duplicates_in_new_feed(make_reader, update_after_one):
+    reader = make_reader(':memory:', plugins=['reader.entry_dedupe'])
+    reader._parser = parser = Parser()
+
+    reader.add_feed(parser.feed(1))
+    one = parser.entry(1, 1, title='title', summary='summary')
+
+    if update_after_one:
+        reader.update_feeds()
+        parser.entries[1].clear()
+        reader.mark_entry_as_read(one)
+        reader.mark_entry_as_important(one)
+        reader.set_tag(one, 'key', 'value')
+
+    parser.entry(1, 2, title='title', summary='summary')
+    parser.entry(1, 3, title='title', summary='summary')
+
+    # shouldn't fail
+    reader.update_feeds()
+
+    assert [eval(e.id)[1] for e in reader.get_entries()] == [3]
