@@ -26,10 +26,10 @@ from reader._types import FeedData
 from reader._types import FeedUpdateIntent
 
 
-def test_storage_errors_connect(tmpdir):
+def test_storage_errors_connect(tmp_path):
     # try to open a directory
     with pytest.raises(StorageError) as excinfo:
-        Storage(str(tmpdir))
+        Storage(str(tmp_path))
     assert isinstance(excinfo.value.__cause__, sqlite3.OperationalError)
     assert 'while opening' in excinfo.value.message
 
@@ -208,6 +208,14 @@ def mark_as_important(storage, feed, entry):
     storage.mark_as_important(feed.url, entry.id, 1, None)
 
 
+def get_entry_recent_sort(storage, feed, entry):
+    storage.get_entry_recent_sort(entry.resource_id)
+
+
+def set_entry_recent_sort(storage, feed, entry):
+    storage.set_entry_recent_sort(entry.resource_id, datetime(2010, 1, 1))
+
+
 def update_feed(storage, feed, entry):
     storage.update_feed(FeedUpdateIntent(feed.url, entry.updated, feed=feed))
 
@@ -219,7 +227,11 @@ def update_feed_last_updated(storage, feed, entry):
 def add_or_update_entry(storage, feed, entry):
     storage.add_or_update_entry(
         EntryUpdateIntent(
-            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1)
+            entry,
+            entry.updated,
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
         )
     )
 
@@ -232,6 +244,7 @@ def add_or_update_entries(storage, feed, entry):
                 entry.updated,
                 datetime(2010, 1, 1),
                 datetime(2010, 1, 1),
+                datetime(2010, 1, 1),
             )
         ]
     )
@@ -240,7 +253,11 @@ def add_or_update_entries(storage, feed, entry):
 def add_entry(storage, feed, entry):
     storage.add_entry(
         EntryUpdateIntent(
-            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1)
+            entry,
+            entry.updated,
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
         )
     )
 
@@ -300,6 +317,8 @@ def get_entry_last(storage, feed, entry):
         mark_as_stale,
         mark_as_read,
         mark_as_important,
+        get_entry_recent_sort,
+        set_entry_recent_sort,
         update_feed,
         update_feed_last_updated,
         add_or_update_entry,
@@ -335,7 +354,13 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
     storage.add_feed(feed.url, datetime(2010, 1, 1))
     storage.add_or_update_entry(
         EntryUpdateIntent(
-            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1), 0, 0
+            entry,
+            entry.updated,
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            0,
+            0,
         )
     )
 
@@ -429,7 +454,13 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
     storage.add_feed(feed.url, datetime(2010, 1, 2))
     storage.add_or_update_entry(
         EntryUpdateIntent(
-            entry, entry.updated, datetime(2010, 1, 1), datetime(2010, 1, 1), 0, 0
+            entry,
+            entry.updated,
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
+            0,
+            0,
         )
     )
     storage.add_feed('two', datetime(2010, 1, 1))
@@ -439,6 +470,7 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
             entry.updated,
             datetime(2010, 1, 1),
             datetime(2010, 1, 1),
+            datetime(2010, 1, 1),
             0,
             0,
         )
@@ -479,6 +511,7 @@ def storage_with_two_entries(storage):
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
+            datetime(2010, 1, 2),
             0,
             0,
         )
@@ -489,6 +522,7 @@ def storage_with_two_entries(storage):
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
             datetime(2010, 1, 2),
+            datetime(2010, 1, 2),
             1,
             0,
         )
@@ -501,7 +535,7 @@ def test_important_unimportant_by_default(storage):
     assert {
         e.id
         for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=False)
+            datetime(2010, 1, 1), EntryFilterOptions(important='nottrue')
         )
     } == {'one', 'two'}
 
@@ -517,19 +551,19 @@ def test_important_get_entries(storage):
     assert {
         e.id
         for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=None)
+            datetime(2010, 1, 1), EntryFilterOptions(important='any')
         )
     } == {'one', 'two'}
     assert {
         e.id
         for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=True)
+            datetime(2010, 1, 1), EntryFilterOptions(important='istrue')
         )
     } == {'one'}
     assert {
         e.id
         for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=False)
+            datetime(2010, 1, 1), EntryFilterOptions(important='nottrue')
         )
     } == {'two'}
 
@@ -540,7 +574,7 @@ def test_important_entry_important(storage):
 
     assert {e.id: e.important for e in storage.get_entries(datetime(2010, 1, 1))} == {
         'one': True,
-        'two': False,
+        'two': None,
     }
 
 
@@ -552,7 +586,7 @@ def test_important_mark_as_unimportant(storage):
     assert {
         e.id
         for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important=True)
+            datetime(2010, 1, 1), EntryFilterOptions(important='istrue')
         )
     } == set()
 
@@ -590,3 +624,19 @@ def test_migration_minimum_version(db_path, request):
 
     assert 'no migration' in str(excinfo.value)
     assert '://reader.readthedocs.io/en/latest/changelog.html' in str(excinfo.value)
+
+
+@rename_argument('storage', 'storage_with_two_entries')
+def test_get_set_recent_sort(storage):
+    assert storage.get_entry_recent_sort(('feed', 'one')) == datetime(2010, 1, 2)
+
+    storage.set_entry_recent_sort(('feed', 'one'), datetime(2010, 1, 3))
+    assert storage.get_entry_recent_sort(('feed', 'one')) == datetime(2010, 1, 3)
+
+    with pytest.raises(EntryNotFoundError) as excinfo:
+        storage.get_entry_recent_sort(('feed', 'xxx'))
+    assert excinfo.value.resource_id == ('feed', 'xxx')
+
+    with pytest.raises(EntryNotFoundError) as excinfo:
+        storage.set_entry_recent_sort(('feed', 'xxx'), datetime(2010, 1, 1))
+    assert excinfo.value.resource_id == ('feed', 'xxx')
